#ifndef ADL_H
#define ADL_H

#include <pthread.h>

//#include "context.h"
#include "connection.h"
#include "hash_request.h"

#ifndef MOVING_AVERAGE_LEN
#define MOVING_AVERAGE_LEN	4
#endif

#ifndef ADL_DEFAULT_SELECT_TIME
#define ADL_DEFAULT_SELECT_TIME	2
#endif

#ifndef DBUFSIZE
// DBUFSIZE == max tcp recv size
#define DBUFSIZE 1024*128
#endif



typedef struct downloadContext {
	int sock;
	int source;	// flag for whether we are connected to a source or a peer
	pthread_mutex_t * lock;
	connection_t * conn;
	unsigned char dbuf[DBUFSIZE];
	int dbuflen;		// how much of dbuf is filled
	int offset;		// how much has been written to file
	int bytes;	// bytes grabbed since last tick
	int time_start;	// time since last tick
			// throughput= bytes/time_start
	int block, current;	// start,stop, and current amount received
			// position of current job
	double obps;
		// current==stop --> done
} downloadContext;



typedef struct adaptiveContext{
	double max;
	double act;
	double old;
	double avg;
	double std;
	int timer;
	int timer_inc;
	double b_inc;
	int nBandwidthEstimates;
	int indexBandwidthEstimates;
	long * bandwidthEstimates; 
	downloadContext ** connection;
	volatile int nConnections;
	int maxConnections;
	pthread_mutex_t * lock;
	pthread_cond_t *cond;
  
        // A thing to keep track of the hash requests we have made
        struct hashRequest_t* hashReq;

	pthread_mutex_t * inProgressLock;
	blocks * inProgress;
} adaptiveContext;


int adaptiveDownloadFile(struct context *ctx);
adaptiveContext * adaptiveContextNew(struct context *ctx);
int spawnAdaptiveDownloaderThread(struct context * ctx);
int reassignFreeDownloader(struct context * ctx,int i);
int addConnection(struct context * ctx);
int addSourceConnection(struct context * ctx);
int addPeerConnection(struct context * ctx, int block, int count, download **);
int dropConnection(struct context * ctx);
int updateBandwidthEstimates(struct context *ctx);
int adaptiveDoneYet(struct context * ctx);
double getDownloadBps(downloadContext *dlc);

int killConnection(struct context * ctx,int i); // assume ac is locked
int getNewAssignment(struct context * ctx,downloadContext *dlc); // assume ac is locked

int increaseUpdateRate(struct context * ctx);
int decreaseUpdateRate(struct context * ctx);

/* ATOMIC note:
 * 	the inProgress bit field should be locked, but since all ops on it are atomic,
 * 	and it is not the end of the world if things are not correct (an extra block is dl'ed),
 * 	then we don't need to incur the significant over head in locking the entire thing
 * 	while we are searching through it
 */

#endif
