#ifndef BLOCKS_H
#define BLOCKS_H
#include <pthread.h>

typedef struct {
  int blockSize;
  int numBlocks;
  int totalSize;
  unsigned char * bitMap;
  // for synchronization
  pthread_mutex_t *lock;
  int refCount;
} blocks;

blocks * blocksNew(int blockSize, int totalSize);
int blocksIsSet(blocks * b,int block);
int blocksSet(blocks * b,int block);
int blocksClear(blocks *b);
int blocksClearBit(blocks *b,int block);
void blocksFree(blocks *b);
//int blocksMerge(blocks *dst, blocks * a, blocks *b);
// 1 range of bytes is set, 0 otherwise
// range [offset .. offset+len-1]
// useful to see if we already downloaded the requested
// range of bytes
int blocksIsByteRangeSet(blocks *b, size_t offset, size_t len);
blocks * blocksMerge(blocks * a, blocks *b);
int blocksRead(blocks * b,int fd,int timeout);
int blocksWrite(blocks * b,int fd,int timeout);
blocks * blocksDuplicate(blocks *b);
int blocksIncRef(blocks *b);
int blocksFull(blocks *b);
int blocksCountBits(blocks *b);
int blocksCmp(blocks * a, blocks * b);
int do_blocks_tests();



#endif
