#ifndef CONTEXT_H
#define CONTEXT_H

#include <pthread.h>
#include <time.h>
struct context;

#include "neighborManager.h"
//#include "leeches.h"
//#include "downloads.h"
#include "queue.h"
#include "blocks.h"
#include "updateTree.h"
#include "dl.h"
#include "adl.h"
#include "leech_server_new.h"
#include "protocol.h"

#define BLOCKSIZEPRIORITY	0x01
#define NUMBLOCKSPRIORITY	0x02

#ifndef DEFAULTNUMNEIGHBORS
#define DEFAULTNUMNEIGHBORS 10
#endif
#ifndef MAXNUMNEIGHBORS
#define MAXNUMNEIGHBORS 30
#endif

#define DOWNLOADTYPE_DUMB	0x0
#define DOWNLOADTYPE_ADAPTIVE	0x1

#ifndef DEFAULT_ADL_MAX_BANDWIDTH
#define DEFAULT_ADL_MAX_BANDWIDTH	100*1024*1024l
#endif
#ifndef DEFAULT_ADL_TIMER
#define DEFAULT_ADL_TIMER	1
#endif
#ifndef DEFAULT_ADL_TIMER_INC
#define DEFAULT_ADL_TIMER_INC	1
#endif
#ifndef DEFAULT_ADL_MAX_CONNECTIONS
#define DEFAULT_ADL_MAX_CONNECTIONS	64
#endif
#ifndef DEFAULT_ADL_BANDWIDTH_INC
#define DEFAULT_ADL_BANDWIDTH_INC	4096
#endif

#ifndef DEFAULT_HASH_MAX_REQ_SIZE
#define DEFAULT_HASH_MAX_REQ_SIZE       1024         // kilobytes
#endif

#ifndef DEFAULT_MAGIC_TIME
#define DEFAULT_MAGIC_TIME	0
#endif

#ifndef DEFAULT_BACKOFF_TIME
#define DEFAULT_BACKOFF_TIME	4
#endif

#ifndef BACKOFFBIASDEFAULT
#define BACKOFFBIASDEFAULT 3
#endif


#define BACKOFFRULE_NONE	0x0
#define BACKOFFRULE_SAFE	0x1
#define BACKOFFRULE_GOOD	0x3

typedef struct context {
  // Parameters chosen by User
  char * slurpieServerAddress;
  int slurpieServerPort;
  char * URL; // Source URL 
  char * URLMD5SUM; // Source URL for verification purposes
  char * destFileName;	// destination file name
  
  // leech parameters
  leech_server_t *leechServerP;
  //int leechLocalPort; // port for leech connects
  int maxNumOfLeeches; 
  int numOfLeechThreads; // number of leech serving threads
  int useMTLeechServ;    // whether to use the multi-threaded leech server
  unsigned int localIp;
  // do not need those anymore
  //queuetype* leeches; // people who we are uploading to 
  //pthread_t *leechThreadP; // leech server thread
  pthread_t *updateThreadP; // update server thread

  int phase2;			// are we in phase2 of the download?
  int maxNumOfDownloads;
  int maxUpdateTreeLeaves;// number of leaves in the updateTree
  int maxNumOfNeighbors;
  int desiredNumOfNeighbors;	// the number of neighbors we want; < max num of neighbors
  int desiredNumOfUpdates;	// the number of updates we want sent to us each round
  int neighborChoice;
  int blockingPriority;	// which is the independent variable: blocksize or number of blocks
  int pickTwo;
  int randomSeed;

  int backoffRule;
  time_t backOffCommitTime;
  int 	backoffDidBackOff;
  int backoffTime;
  int backoffBias;
  int magicTime;	// time to hang around after we are done
  
  int fileSize;		// in bytes
  int blockSize;		// in bytes
  struct dlqueue * dlq;		// downloading queue
  struct adaptiveContext * ac;	// adaptiveContext;
  int quiet;		// flag for whether we have output or not
  int downloadType;
  
  time_t startTime, stopTime;

  int peerBlocksDld, sourceBlocksDld;
  
  // Private State
  int blockRule; 
  int numberOfBlocks;	// ceil(fileSize/blockSize)
  int fd;			// actual file destination
  
  // Data Structures
  blocks * dlBitMap;	// bit map of what we have downloaded so 
  // far including finished blocks on tmpDir
  struct neighborManager* neighborMgr; // The Mesh manager
  // not used; moved into qlQueue struct
  // queuetype* downloads; // people who we are downloading *from*
  
  struct UpdateTree * updateTree;	// what the update thread
  // updates
  // Parameters for adaptive Downloading
  double adl_maxBandwidth;
  int adl_timer;
  int adl_timer_inc;
  int adl_maxConnections;
  int adl_bandwidth_inc;
  
  // Hash checker
  int hash_maxRequestSize;
  int hash_enabled;
  struct hashCheck_t* hashChecker;

  // Synchronization Primitives
  pthread_mutex_t* lock;	
  pthread_mutex_t* flock;	
} context;

// The context structure creator. Takes as a parameter a preallocated structure
// and the URL for the file that we want to download.
context* contextCreate();

// The context destructor. You should only call this if the context state is
// is disconnected
void contextDestroy(context* c);

// Sets the User parameters based on argc and argv
// Returns 0 if no error -1 if something bad happened.
int contextParseParameters(context* c, int argc, char** argv);

// Prints to stdout the correct usage of parameters
void usage(char *,char*);

// Starts listening for neighbor connects
void contextCreateNeighborServer(context* c);

// Starts listening for leech connects 
void contextCreateLeechServer(context* c);

void contextCreateHashCheck(context* c);

// Slurpie Join. This will contact the slurpie server and download a list of
// neighbors. 
int contextJoin(context* c);

// opens the DST file and returns the fd, after setting it in context
int contextFileOpen(context *c);

char * contextParseFileName(context *c);

void do_context_tests();
#endif
