#ifndef FTP_UTILS_H
#define FTP_UTILS_H

#ifndef IPPORT_FTP
#define IPPORT_FTP 21
#endif

/* Interface for FTP protocol primitives */

#define FTP_TIMEOUT 60*1000  // 60 seconds

#define FTP_MAX_LINE 128
#define FTP_RETCODE_SIZE 3

// return codes from functions in that module
#define FTP_TCP_ERR -1
#define FTP_FTP_ERR -2
#define FTP_CMD_OK   0


// server return codes from RFC 959
// expected result after establishing TCP connection to server
#define FTP_SERVER_READY 220
// expected result for USER and PASS command 
#define FTP_USER_LOGGED 230
// expected result of PASV command
#define FTP_PASV_MODE_OK 227
// expected result of TYPE command
#define FTP_TYPE_OK 200
// expected result of REST command
#define FTP_REST_OK 350 
// expected result of RETR command
#define FTP_FILE_REQ_OK 150

int ftp_connect_cmd(int theControlSock);
int ftp_login_cmd(int theControlSock, 
		  const char* theUser, 
		  const char* thePass);
int ftp_retr_cmd(int theControlSock, 
		 const char theType,         // data transmission type I binary, A ascii
		 const char* theFileName, 
		 unsigned int theOffset,
		 int *theDataSockOut);// data socket where the data will be output
int ftp_list_cmd(int theControlSock,
		 const char* theFileGrpNameP, // either name of directory or file
		 int *theDataSockP); // data socket where the listing will be output
// will parse the list command output
// only interested in file size now
// NOTE: the output is not standardized therefore can be problems here
// Assume the outpute is in form <permissions> <number> <string> <string> <size> ...
int ftp_parse_list_out(int theDataSock,
		       unsigned long *theSizeP);
#endif
