#ifndef _HASH_REQUEST_H
#define _HASH_REQUEST_H


#include <pthread.h>
#include <openssl/evp.h>
#include "context.h"
#include "queue.h"
#include "connection.h"

#ifndef DEFAULT_HASH_VERIFICATIONS
#define DEFAULT_HASH_VERIFICATIONS        3
#endif

#ifndef MAX_HASH_CONNECTIONS
#define MAX_HASH_CONNECTIONS        50
#endif

typedef struct hashElem_t {
  int answered;                    // Whether this host has given me a hash yet
  unsigned int addr;                 // The addr I sent this request to
  unsigned short port;               // The port I sent this request to
  char their_hash[EVP_MAX_MD_SIZE];  // The hash this host computed
  struct hashElem_t* next;
} hashElem_t;

typedef struct hashHead_t {
  char my_hash[EVP_MAX_MD_SIZE];
  unsigned int src_addr;
  unsigned short src_port;
  int offset, len;
  struct hashElem_t* elem;
} hashHead_t;

typedef struct hashInfo_t {

  connection_t con;
  hashHead_t* hash;
} hashInfo_t;

typedef struct hashRequest_t {
  
  const EVP_MD* evp_md;       // For accessing the hashing functions
  struct context* ctx;        // Everybody loves the context...
  unsigned char numVerif;     // How many people we need to verify each block
  pthread_t* hashThread;      // Thread to send hash requests
  pthread_mutex_t* mutex;     // A mutex for the queue below
  pthread_cond_t* cond;       // For sleeping on empty queue
  queuetype* inputQueue;       // A queue of the blocks we want requests for
  hashInfo_t* reqs[MAX_HASH_CONNECTIONS];  // Current outgoing connections
  int numReqs;
  fd_set sockets;             // A set for select()
  int maxFD;                  // The maximum numbered socket
  
} hashRequest_t;

hashRequest_t* hash_request_create(struct context* ctx);
int hash_request_make_request(hashRequest_t* hasher, hashHead_t* req, int sock);
int hash_request_send_requests(hashRequest_t* hasher, hashHead_t* req,
			      download** dst, int num);
void hash_request_run_thread(hashRequest_t* hasher);

#endif // _HASH_REQUEST_H
