#ifndef LEECH_PROTOCOL_H
#define LEECH_PROTOCOL_H

#include "version.h"
#include "connection.h"

/*  This file contains all the definitions for the download protocol between peers. */
/*  Here is the specification of the protocol: */

/*  CLIENT: GET <offset> <length> <URL> CRLF */
/*  SERVER: <ret code> <string> */
/*  if red code == LEECH_CMD_OK */
/*  SERVER: <bytes requested> */

#define LEECH_DEFAULT_PORT 33456
#define LEECH_GET_STR "GET"

// Ryan: added this for the hash request, maybe it shouldn't be here?
#define HASH_GET_STR  "HASH"

#define LEECH_TIMEOUT 10*1000 // 10 seconds
#define LEECH_MAX_LINE 4096
#define LEECH_EOL "\n"

// server return codes
// command is OK file is availabe and will be served to the client
#define LEECH_CMD_OK 200
#define LEECH_CMD_OK_STR "Command OK"
// the named file is not available
#define LEECH_FILE_UNAVAIL 404
#define LEECH_FILE_UNAVAIL_STR "Requested file is unavailable"
// the named file is available but the requested range of bytes is not available
#define LEECH_BYTES_UNAVAIL 416
#define LEECH_BYTES_UNAVAIL_STR "Requested bytes are unavailable"
// the service is not available the maximum number of clients was reached
#define LEECH_SERVICE_UNAVAIL 503
#define LEECH_SERVICE_UNAVAIL_STR "Maximum number of clients reached"
// the client is closing the persistent connection
#define LEECH_BYE 1
#define LEECH_BYE_STR "Client disconnected"
#define LEECH_BYE_WORD "BYE"
#define LEECH_BYE_MSG "BYE BYE BYE"

// return codes from functions
#define LEECH_TCP_ERR -2
#define LEECH_ERR     -1
#define LEECH_OK       0

// will send retcode specified on the connection
int leech_send_retcode(connection_t *theClientConnectionP,
		       unsigned int theRetcode,
		       const char *theRetcodeStr);
// will read the return code
int leech_get_retcode(connection_t *theServerConnP);

// will send command
int leech_send_cmd(connection_t *theServerConnP,
		   const char *theUrlBufP,
		   size_t theOffset,
		   size_t theLen);

// will read the command from connection and parse it
int  leech_get_cmd(connection_t *theConnectionP,
		   char *theUrlBufOutP,
		   size_t theUrlBufLen,
		   size_t *theOffsetOutP,
		   size_t *theLenOutP);

#endif
