#ifndef NEIGHBORMANAGER_H
#define NEIGHBORMANAGER_H

#include <pthread.h>

struct neighborManager;

#include "connection.h"
#include "queue.h"
#include "neighbor.h"
#include "context.h"
#include "updateTree.h"

// This is basically the narada mesh manager
// It should listen to new connections, drop ones that seems dead, 
// provide an API to publish data and to get data out of the mesh
//

#ifndef NEIGHBORMANAGER_TIMEOUT
#define NEIGHBORMANAGER_TIMEOUT	3*1000
#endif

typedef struct neighborManager {
	// Sync primitives
	pthread_mutex_t* lock;

	// Chosen by User
	int maxNumOfNeighbors; 	// max possible
	int nNeighbors;		// current number

	// Everything below here is Private
	connection_t* daemon;
	neighbor ** neighbors; // connections to other peers for topology links

	// More sync primitives and Thread information
	pthread_cond_t* condNumOfNeighbors;
	pthread_cond_t* condNewNeighbor;
	pthread_t acceptThread;
	pthread_t managerThread;

	// A ref count and a state for safe thread cancelation
	unsigned short state; // 0 destroy, 1 running
	unsigned short refCount;
	struct context * ctx;

} neighborManager;

// Creates the server.
neighborManager* neighborManagerCreate(struct context *);

// Disconnects from the Topology graph
void neighborManagerDestroy(neighborManager* n);

// Sets the new maxNumOfNeighbors. 
// Returns 0 on success and -1 on error
int neighborManagerSetMaxNumOfNeighbors(neighborManager* n, unsigned short neo);

// Remove a peer from neighbor manager's neighbor list.
// Return non zero on failure, zero otherwise.
int neighborManagerRemovePeer(neighborManager *nm, neighbor *n);

// Returns the addr that the NManager is listening at
const char* neighborManagerGetAddr(neighborManager*, char* addr, size_t len);
// Gets the port the NManager is listening at
unsigned short neighborManagerGetPort(neighborManager*n);

// Tries to add a new peer
// Return 0 on success, -1 if it fails
int neighborManagerAddPeer(neighborManager* n, connection_t* neo);
int neighborManagerConnectToPeer(neighborManager* n, const char* host, unsigned short port);

// Emulates select(): 
// 	returns the number of neihbors who have something to say
// 	zero on timeout, negative on error

int neighborManagerSelect(neighborManager* n, neighbor **list, int listlen);

// exports a list of the neighbors at the current time

int neighborManagerGetList(neighborManager *, neighbor ** list, int listlen);

// Needs some sort of write and read
// int neighborManagerWrite(...)
// int neighborManagerRead(...)

// Pick someone random from our updateTree and try to add them
int addRandomNeighbor(neighborManager *n);


// Unit Test
void do_neighborManager_tests();

#endif
