#ifndef UPDATETREE_H
#define UPDATETREE_H

#include <pthread.h>


struct UpdateTree;
#include "blocks.h"
#include "downloads.h"
#include "updates.h"

typedef struct UpdateNode{
	struct UpdateNode *left, *right,*next;
	unsigned short revoked; // 0 is good 1 is revoked
        unsigned short busy;  // 0 is not busy, 1 is busy
	//struct UpdateNode * parent;
	blocks * data;
	unsigned int ip;
	unsigned short port;
} UpdateNode;

typedef struct UpdateTree {

	UpdateNode * root;
	int maxLeaves;
	int numLeaves;
	UpdateNode ** leaves;
	pthread_mutex_t *lock;

} UpdateTree;



UpdateTree * updateTreeNew(int maxLeaves);
int updateTreeFree(UpdateTree *);

/*  // updateTreeUpdate and updateTreeAdd have been merged into the same 
	functions
int updateTreeAdd(UpdateTree *, unsigned int ip, unsigned short port,
		blocks *b); 

*/
int updateTreeUpdate(struct context *, UpdateTree *, unsigned int ip, unsigned
		short port, blocks *b);

int updateTreeSearch(UpdateTree *, int block, int * count, download ** dls);
// Find a block that is not downloaded, that we have to get from the source
// 	return negative number on error, or if one doesn't exist
int updateTreeFindMissingBlock(UpdateTree *ut,struct context *c);
int updateTreeFindMissingBlockRange(UpdateTree *ut,struct context *c, int start, int stop);
// is this block in the updateTree?
int updateTreeExists(UpdateTree *ut,int block);
// Revoke all blocks from ip/port
int updateTreeRevoke(UpdateTree* ut, unsigned int ip, unsigned short port);

int printStatus(struct context *);
int printAdaptiveStatus(struct context *);
int printAdaptiveStatusLine(struct context *);
int updateTreeCountNodes(struct context *c);
struct update * getRandomGoodNeighborUpdate(struct context *c);

int updateTreeMarkLeafNotBusy(struct context *c, connection_t* con);
int updateTreeMarkLeafBusy(struct context *c, download *dl);


/****************************************************************************
 *  private functions
 ***************************************************************************/

int updateTreeSearch2(UpdateNode *n, int block, int max, int curr, download ** dls);


void updateTreeProcessUpdate(UpdateTree *ut);
int updateTreeTearDownTree(UpdateTree * ut);
int do_updateTree_tests();
int updateTreeTearDownTree2(UpdateNode * );
int getRandomUpdates(UpdateTree *,struct context *c, struct update ** ulist, int max);
int updateTreeFirstNeedBlock(struct context * ctx,unsigned int ip,unsigned short port,int start);



#endif
