#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "backoff.h"
#include "utils.h"
#include "updateTree.h"


/*
int backoff1or2(context * ctx){
	int count;
	count = updateTreeCountNodes(ctx);
	if((!ctx->backoffRule)||(count<BACKOFF_MIN)){
		ctx->didBackOff=0;
		return 0;
	}
	if(ctx->backoffDidCommit>0){	// we are already backed off
		if(time(NULL)>=ctx->didBackOff){	// test if we should go back
			Dprintf("Unbacking off\n");
			ctx->didBackOff=0;
			if(ctx->backoffRule==2)
				return backoff(ctx);	// try again
			else
				return 0;		// immeadiately succeed
		}
		else return 1;			// else we are still backed off
	}
	// not backed off
	if(!(abs(mrand48())%(count))){	// a 1 in n chance, normally
		Dprintf("DIDN'T BACK OFF: %d\n",count);
		ctx->didBackOff=0;
		return 0;
	}
	Dprintf("BACKED OFF!\n");
	ctx->didBackOff=time(NULL)+ctx->backoffTime;
	return 1;
}
*/

/* when we do a backoff check for reassignment, bias things so that people who are reassigned
 * blocks are much more likely
 */

/*
int backoff1or2_reassign_bias(context * ctx){
	int count;
	count = updateTreeCountNodes(ctx);
	if((!ctx->backoffRule)||(count<BACKOFF_MIN)){
		ctx->didBackOff=0;
		return 0;
	}
	if(ctx->didBackOff>0){	// we are already backed off
		if(time(NULL)>=ctx->didBackOff){	// test if we should go back
			Dprintf("Unbacking off\n");
			ctx->didBackOff=0;
			if(ctx->backoffRule==2)
				return backoff_reassign_bias(ctx);	// try again
			else
				return 0;		// immeadiately succeed
		}
		else return 1;			// else we are still backed off
	}
	// not backed off
	if(!(abs(mrand48())%(count/BACKOFF_REASSIGN_BIAS))){	// a BACKOFF_REASSIGN_BIAS in n chance, normally
		Dprintf("DIDN'T BACK OFF (BIASED) : %d\n",count);
		ctx->didBackOff=0;
		return 0;
	}
	Dprintf("BACKED OFF!\n");
	ctx->didBackOff=time(NULL)+ctx->backoffTime;
	return 1;
}
*/

/* backoff rule 3:
 * 	if you backoff, backoff for time ctx->backoffTime
 * 	if you commit, then *commit* for the same amount of time
 * 		which is okay, b/c further calls to backoff will 
 * 		be backoff_reassign_bias instead
 */

/* c->backOffCommitTime=DEFAULT_BACKOFF_TIME;
 *   c->backoffDidCommit=0;
 */
int backoff3(context * ctx){
	int count;
	count = updateTreeCountNodes(ctx);
	if(count<BACKOFF_MIN){
		ctx->backoffDidBackOff=0;	// don't backoff unless there are some people in the
					// system
		return 0;
	}
	if(time(NULL)<ctx->backOffCommitTime){	// test if we are still in our time window
		Dprintf("BACKOFF: still %s\n",ctx->backoffDidBackOff?"backed off":"commited");
		return ctx->backoffDidBackOff;
	}
	// else try our luck again
	ctx->backOffCommitTime=time(NULL)+ctx->backoffTime;
	if(!(abs(mrand48())%(MAX(1,count/ctx->backoffBias)))){	// a ctx->backoffBias in n chance, normally
		Dprintf("DIDN'T BACK OFF: %d\n",count);
		ctx->backoffDidBackOff=0;
		return 0;
	}
	Dprintf("BACKED OFF!\n");
	ctx->backoffDidBackOff=1;
	return 1;	// else backoff
}

/*

int backoff3_reassign_bias(context * ctx){
	int count;
	count = updateTreeCountNodes(ctx);
	if(count<BACKOFF_MIN){
		ctx->didBackOff=0;	// don't backoff unless there are some people in the
					// system
		return 0;
	}
	if(ctx->didBackOff>0){	// we are already backed off
		if(time(NULL)>=ctx->didBackOff){	// test if we should go back
			Dprintf("Unbacking off bias\n");
			ctx->didBackOff=0;
			return backoff3(ctx);	// try again
		}
		else return 0;			// else we are still committed!
	}
	// not backed off
	ctx->didBackOff=time(NULL)+ctx->backoffTime;
	if(!(abs(mrand48())%(MAX(1,count/ctx->backoffBias)))){	// a ctx->backoffBias in n chance, normally
		Dprintf("DIDN'T BACK OFF bias: %d\n",count);
		return 0;
	}
	Dprintf("BACKED OFF! bias\n");
	return 1;	// else backoff
}

*/
int backoff(context * ctx){
	switch(ctx->backoffRule){
		case 0: 
			ctx->backoffDidBackOff=0;
			return 0;
		/*
		case 1:
		case 2:
			return backoff1or2(ctx);
			*/
		case 3:
			return backoff3(ctx);
		default: 	
			abort();
	}
	abort();
	return 0;	// should never reach
}
int backoff_reassign_bias(context * ctx){
	switch(ctx->backoffRule){
		case 0: 
			ctx->backoffDidBackOff=0;
			return 0;
			/*
		case 1:
		case 2:
			return backoff1or2_reassign_bias(ctx);
			*/
		case 3:
			//return backoff3_reassign_bias(ctx);
			return backoff3(ctx);
		default: 	
			abort();
	}
	abort();
	return 0;	// should never reach
}
