#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
// 

#include "dl.h"
#include "context.h"
#include "updateTree.h"
#include "downloads.h"
#include "source_get.h"
#include "leech_utils.h"
#include "leech_get.h"
#include "leech_protocol.h"
#include "utils.h"

int getHostRank(download* ip);
int setHostRank(download* ip,int);

void * peerDownloader(void *);
void * sourceDownloader(void *);

dlqueue * dlqueueNew(context *c, int nThreads);
int doneYet(context *c);


int spawnSourceDownloader(context *c);

// putting block requests out for the leech threads to grab
int dlEnqueue(context *c, download * d, int block);
int dlDequeue(context *c, download * d, int * block);

download * dlPickBestHost(context *c, download **list, int count);


/* int downloadFile(context *c) {
 * 	actual call to start the magic et al
 */

int downloadFile(context *c) {
  int i;
  download ** list;
  download * best;
  int count;
  if(c->downloadType==DOWNLOADTYPE_ADAPTIVE)
	  return adaptiveDownloadFile(c);
  Dprintf("downloadFile ENTERED!\n");
  list = malloc(sizeof(download *)*DL_MAXNODESPERBLOCK);
  if(!list){
    perror("downloadFile::malloc");
    abort();
  }

  c->dlq = dlqueueNew(c,c->maxNumOfDownloads);
  spawnSourceDownloader(c);
	while(!doneYet(c)){
		for(i=0;i<c->numberOfBlocks;i++){	// start peer downloading from the from
			// see note in dl.h
			if(blocksIsSet(c->dlBitMap,i)||
					blocksIsSet(c->dlq->inProgress,i))
				continue;	// we are already working on it
			// see NOTE 2 on locking
			count = DL_MAXNODESPERBLOCK;
			{
				int j = 0;
				for (j = 0; j < count; j++)
					list[j] = 0;
			}
			count = updateTreeSearch(c->updateTree, i, &count, list);
			if(count<1)	// no one found :(
				continue;
			// Dprintf("POSSIBLE PEERS %d\n", count);
			best = dlPickBestHost(c,list,count);
			dlEnqueue(c,best, i);
		}

	}

	// When finished downloading, inform the server I'm done.
	Dprintf("downloadFile ENDED\n");
	return 0;
}

/* int getHostRank(download* ip)
 * 	used for remembering better hosts
 *	NOT used currently
 */
int getHostRank(download* ip)
{
	ip=ip;
	/* For now, just return a constant value. */
	return DL_DEFAULTNODERANK;
}

/* int setHostRank(download *,i):
 * 	remember this host has having priority i
 */

int setHostRank(download *ip,int i){
	ip=ip;
	i=i;
	return 0;
}

/* int doneYet(c):
 * 	<whine>... are we done yet?
 */
int doneYet(context *c){
	if(c->numberOfBlocks<=c->dlq->blocksDoneCount)	
		return 1;	// DONE!
	return 0;	// not :(
}

/* dlqueue * dlqueueNew(context *c, int nThreads);
 * 	make a new context, spawn threads for it, but they 
 * 	are locked, cuz they don't have anything to do
 */
dlqueue * dlqueueNew(context *c, int nThreads){
	dlqueue *ret;
	int err;
	int i;

	ret = malloc(sizeof(dlqueue));
	if(!ret){
		perror("dlqueueNew::malloc");
		assert(ret);
	}
	c->dlq=ret;
	ret->lock = malloc(sizeof(pthread_mutex_t));
	ret->cond = malloc(sizeof(pthread_cond_t));
	if(!ret->lock || !ret->cond){
		perror("dlqueueNew::malloc");
		abort();
	}
	pthread_mutex_init(ret->lock,NULL);
	pthread_cond_init(ret->cond,NULL);

	ret->blocks = Q_Create();
	//ret->ips = Q_Create();
	ret->threadCount = nThreads;
	ret->blocksDoneCount=0;
	ret->inProgress = blocksNew(c->blockSize,c->fileSize);
	ret->threads = malloc(sizeof(pthread_t)*nThreads);
	if(!ret->threads){
		perror("dlqueueNew::malloc");
		assert(ret->threads);
	}
	for(i=0;i<nThreads;i++){
		err = pthread_create(&ret->threads[i],NULL,peerDownloader,c);
		if(err){
			perror("dlqueueNew::pthread_create");
			abort();
		}
		pthread_detach(ret->threads[i]);
	}

	return ret;
}

/* spawnSourceDownloader(context *c):
 * 	start off the source downloader thread
 */

int spawnSourceDownloader(context *c){
	pthread_t * thread;
	int err;

	thread = malloc(sizeof(pthread_t));
	if(!thread){
		perror("spawnSourceDownloader::malloc");
		assert(thread);
	}
	err = pthread_create(thread, NULL, sourceDownloader,c);
	if(err){
		perror("spawnSourceDownloader::pthread_create");
		abort();
	}
	pthread_detach(*thread);
	return 0;
}

/* download * dlPickBestHost(context *c, download **list, int count);
 * 	pick the best host from the list, breakking ties randomly.
 * 	Since right now we don't implement the host ranking, everything
 * 	is a tie, so this is just a random picker in effect
 */

download * dlPickBestHost(context *c, download **list, int count){
	int max,tmp;
	int i;
	download* best=NULL;
	int iBest;

	max=-1;
	for(i=0;i<count;i++){
		tmp= getHostRank(list[i]);
		if(tmp<max)
			continue;
		if(tmp>max){
			max=tmp;
			best = list[i];
			iBest = i;
			continue;
		}
		// equal, so flip a coin based on how far we are into the list
		// this way we don't bias towards the end of the list
		//  QUESTION: does this really work?
		//  ANS: yes, but only when they are all the same, suboptimal in general
		if((abs(mrand48())%i)==0){
			max=tmp;
			best = list[i];
			iBest = i;
		}
	}
	// Dprintf("PICKED %d\n", iBest);

	return best;
}



/* void * sourceDownloader(void *);
 * 	while(notdone) dl stuff that we aren't already working on
 *
 */

void * sourceDownloader(void * contextPtr){
	context *c = contextPtr;
	int start,i;
	time_t timestart,timestop;
	char *buf;
	int buflen;
	int err;
	int len;

	buflen=c->blockSize;		// 
	buf = malloc(buflen);
	if(!buf){
		perror("sourceDownloader::malloc");
		abort();
	}
	while((!doneYet(c))){
		pthread_mutex_lock(c->dlq->lock);
		i=start = abs(mrand48())%c->numberOfBlocks;
		i++;
		i=i%c->numberOfBlocks;
		while(1){		// find a block
			if( start==i){	// return if we wrap; everything is being worked on
				sleep(1);
				break;
			}
			if(!blocksIsSet(c->dlBitMap,i) &&	// this block is OK
					!blocksIsSet(c->dlq->inProgress,i) &&
					!updateTreeExists(c->updateTree,i)){
				break;
			}
			i++;	// otherwise move on to the next block
			i=i%c->numberOfBlocks;

		}
		blocksSet(c->dlq->inProgress,i);
		pthread_mutex_unlock(c->dlq->lock);
		timestart=time(NULL);
		if(i == (c->numberOfBlocks-1))	// last block
			len = c->fileSize - ((c->numberOfBlocks-1)*(c->blockSize));
		else 
			len = c->blockSize;
		err = source_get(c,buf,buflen,i*c->blockSize,len);
		timestop=time(NULL);
		if(err){
			blocksClearBit(c->dlq->inProgress,i);
			fprintf(stderr,"Error in contacting source;"
					" continuing sourceDownloader thread\n");
			continue;
		}
		leech_file_write(c,buf,i*c->blockSize,len);
		pthread_mutex_lock(c->dlq->lock);
		blocksClearBit(c->dlq->inProgress,i);
		blocksSet(c->dlBitMap,i);
		c->dlq->blocksDoneCount++;
		updateTreeUpdate(c,c->updateTree,c->localIp, 
				neighborManagerGetPort(c->neighborMgr), blocksDuplicate(c->dlBitMap));
		pthread_mutex_unlock(c->dlq->lock);

	}
	// TODO: Implement stage2 where we just d/l anything that isn't going fast enough

	Dprintf("Download thread EXITING\n");
	free(buf);
	return NULL;
}


/*void * peerDownloader(void *);
 * 	snag stuff from the dlq, dl it, mark it as done, repeat
 */

void * peerDownloader(void * contextPtr){
	context *c=contextPtr;
	download d;
	int block;
	char *buf;
	int buflen;
	int err;
	int len;

	err=0;
	buflen = c->blockSize;
	buf = malloc(buflen);
	if(!buf){
		perror("peerDownloader::malloc");
		assert(buf);
	}
	while(1){
		dlDequeue(c,&d,&block);
		if(block == (c->numberOfBlocks-1))  // last block
			len = c->fileSize - ((c->numberOfBlocks-1)*(c->blockSize));
		else
			len = c->blockSize;
		err= leech_get(c,d.ip,d.port,block*c->blockSize,len);	
		if (err&& err != LEECH_SERVICE_UNAVAIL) {
			updateTreeRevoke(c->updateTree, d.ip, d.port);
		}
		pthread_mutex_lock(c->dlq->lock); {
			blocksClearBit(c->dlq->inProgress,block);
			// For each one of the elems on the queue 
			// take it away from the queue if the ip and port are the
			// ones from the revoked server
			// TODO Come back and decide if this can be eliminated
			if (err && err != LEECH_SERVICE_UNAVAIL) {
				/*
				 DO NOTHING FOR NOW
				dlelm* n = NULL;
				dlelm* first = NULL;
				while ((n = (dlelm*)Dequeue(c->dlq->blocks)) && (n != first)) {
					if (n->ip == d.ip && n->port == d.port) {
						blocksClearBit(c->dlq->inProgress,n->block);
						free(n);
					}
					else {
						if (!first)
							first = n;
						Enqueue(c->dlq->blocks,n);
					}
				}
				*/
			}
			else {
				blocksSet(c->dlBitMap,block);
				c->dlq->blocksDoneCount++;
			}
		} pthread_mutex_unlock(c->dlq->lock);

	}
	free(buf);
	return NULL;
}

/* int dlEnqueue(context *c, download * d, int block);
 *
 * 	add stuff to the download list
 */

int dlEnqueue(context *c, download * d, int block){
	// lock, if the queue is not empty, wait
	// add thinger to queue, unless that ip is
	// 	already there, or that block is already
	// 	being worked on
	// mark it as being worked on
	// unlock, signal, return
	dlelm *elm;
	elm = malloc(sizeof(dlelm));
	if(!elm){
		perror("dlEnqueue:: malloc");
		assert(elm);
	}
	elm->ip = d->ip;
	elm->port = d->port;
	elm->block = block;
	pthread_mutex_lock(c->dlq->lock);
	while(!Q_empty(c->dlq->blocks))
		pthread_cond_wait(c->dlq->cond, c->dlq->lock);	// wait
	if(blocksIsSet(c->dlq->inProgress,block)
			|| blocksIsSet(c->dlBitMap,block)
			// || Q_Contains(c->dlq->ips)	// not implemented yet
		){
		free(elm);
		pthread_mutex_unlock(c->dlq->lock);
		return -1;
	}
	blocksSet(c->dlq->inProgress,block);
	Enqueue(c->dlq->blocks,elm);
	pthread_mutex_unlock(c->dlq->lock);
	pthread_cond_signal(c->dlq->cond);	// notifyAll() :) Only added one guy... no need to wake 11 threads
	return 0;
}

/* int dlDequeue(context *c, download * d, int * block);
 * 	remove stuff from the dl list
 */
int dlDequeue(context *c, download * d, int * block){
	// stub
	//
	// lock, and if queue is empty, wait
	// remove thinger from queue
	// unlock, signal, return
	dlelm * elm;

	pthread_mutex_lock(c->dlq->lock);
	while(Q_Empty(c->dlq->blocks))
		pthread_cond_wait(c->dlq->cond, c->dlq->lock);	// wait
	elm =Dequeue(c->dlq->blocks);
	assert(elm);
	d->ip = elm->ip;
	d->port = elm->port;
	*block = elm->block;
	free(elm);

	pthread_cond_broadcast(c->dlq->cond);	// something got removed
	pthread_mutex_unlock(c->dlq->lock);
	return 0;
}
