#include <source_get.h>
#include <leech_protocol.h>
#include <leech_utils.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>

#include "utils.h"
#include "leech_protocol.h"
#include "connection.h"
#include "leech_get.h"

int leech_get(context *ctx, unsigned int ip, unsigned short port, unsigned long offset, unsigned long length)
{
  int err, status, count, retcode ;
  char ipbuf[BUFLEN];
  char *buf;
  connection_t *conn;

  // connect to ip:port..
  conn = connection_create(PROTO_LEECH, SLURPIE_VERSION);
  inet_ntop(AF_INET,&ip,ipbuf,BUFLEN);
  err = connection_connect(conn, ipbuf, port);	// use funky connect calls
  if(err){
	  Dprintf("leech_get:: %s:%d bad connection\n", ipbuf, port);
	  connection_destroy(conn);
	  return -1;
  }
  err =leech_send_cmd(conn,ctx->URL,offset,length);
  if(err){
	  Dprintf("leech_get:: leech_send_cmd: %s:%d badness \n", ipbuf, port);
	  connection_destroy(conn);
	  return -1;
  }

  retcode = leech_get_retcode(conn);
  /*

  // Send the command!
  count = snprintf(tmpbuf, BUFLEN, "%s %lu %lu %s%s", LEECH_GET_STR, offset, length, ctx->URL, CRLF);
  err = timeout_writeall(&status, sock, tmpbuf, count, LEECH_TIMEOUT);
  if (err)
    return err;
  if (status < count)
    return status - count;

  // Read the reply
  err = timeout_read_line(&status, sock, tmpbuf, BUFLEN, LEECH_TIMEOUT);
  if (err)
    return err;
  if (status < 0)
    return status;
  if (status == 0)
    return -1;
  sscanf(tmpbuf, "%d", &retcode);
  */
  switch (retcode) {
  case LEECH_CMD_OK:
    break;
  case LEECH_FILE_UNAVAIL:
    Dprintf("File unavailable.\n");
    connection_destroy(conn);
    return LEECH_FILE_UNAVAIL;
    break;
  case LEECH_BYTES_UNAVAIL:
    Dprintf("Bytes unavailable.\n");
    connection_destroy(conn);
    return LEECH_BYTES_UNAVAIL;
    break;
  case LEECH_SERVICE_UNAVAIL:
    Dprintf("Service unavailable.\n");
    connection_destroy(conn);
    return LEECH_SERVICE_UNAVAIL;
    break;
  default:
    Dprintf("Unknown error from peer.\n");
    connection_destroy(conn);
    return -1;
    break;
  }

  buf = (char *) malloc(length + 1);
  if (!buf){
	  perror("leech_get::malloc");
	    connection_destroy(conn);
	    return -1;
  }
  err = timeout_readall(&status, connect_get_fd(conn), buf, length, LEECH_TIMEOUT);
  if (err){
	  Dprintf("leech_get:: timeout_readall: error getting data: err=%d,status=%d\n",
			  err,status);
	    connection_destroy(conn);
    	return err;
  }
  Dprintf("leech_get DONE: %s:%d bytes: %lu-%lu\n", inet_ntop(AF_INET,&ip,ipbuf,BUFLEN), 
		  port, offset, offset + length - 1);

  count = leech_file_write(ctx, buf, offset, length);
  if (count < length) {
    Dprintf("leech_file_write:: Written less bytes than asked for!");
    return count - length;
  }

  free(buf);
    connection_destroy(conn);

  return 0;
}
