#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "utils.h"
#include "leech_protocol.h"

int leech_send_retcode(connection_t *con,
		 unsigned int theRetcode,
		 const char *theRetcodeStr) {
  char tempBuf[LEECH_MAX_LINE];
  int count, status, err;
  
  count = snprintf(tempBuf, LEECH_MAX_LINE, "%d %s %s", 
		   theRetcode, 
		   theRetcodeStr, 
		   LEECH_EOL);
  if(con->proto == PROTO_LEECH || con->proto == PROTO_LEECH_PERSIST)
    Dprintf2(3,"LEECH SERVER sending ret: %s", tempBuf);
  else if(con->proto == PROTO_HASH_CHECK)
    Dprintf2(3, "HASH CHECKER sending ret: %s", tempBuf);
  err = timeout_writeall(&status, con->sock, tempBuf, count, LEECH_TIMEOUT);
  
  if (err) {
    return LEECH_TCP_ERR;
  }
  else {
    return LEECH_OK;
  }
}

int leech_get_retcode(connection_t *theServerConnP) {
  char tempBuf[LEECH_MAX_LINE];
  int err, status;

  err = timeout_read_line(&status, 
			  theServerConnP->sock, 
			  tempBuf,
			  LEECH_MAX_LINE, 
			  LEECH_TIMEOUT);
  
  //Dprintf(3, "LEECH CLIENT received: %s\n", tempBuf);

  if (err) {
    return LEECH_TCP_ERR;
  }
  else {
    return atoi(tempBuf);
  }
}

int leech_send_cmd(connection_t *con,
		   const char *theUrlBufP,
		   size_t theOffset,
		   size_t theLen) {
  char tempBuf[LEECH_MAX_LINE];
  int count=0, status, err;

  if(con->proto == PROTO_LEECH || con->proto == PROTO_LEECH_PERSIST) {
    count = snprintf(tempBuf, LEECH_MAX_LINE, "GET %d %d %s%s", 
		     theOffset, 
		     theLen,
		     theUrlBufP, 
		     LEECH_EOL);
    Dprintf2(3,"LEECH CLIENT sending: %s", tempBuf);
  }
  else if(con->proto == PROTO_HASH_CHECK) {
    count = snprintf(tempBuf, LEECH_MAX_LINE, "HASH %d %d %s%s",
		     theOffset,
		     theLen,
		     theUrlBufP,
		     LEECH_EOL);
    Dprintf2(3,"HASH CHECKER sending: %s", tempBuf);
  }
  err = timeout_writeall(&status, 
			 con->sock, 
			 tempBuf, 
			 count, 
			 LEECH_TIMEOUT);
  if (err) {
    return LEECH_TCP_ERR;
  }
  else {
    return LEECH_OK;
  }
}

int  leech_get_cmd(connection_t *con,
		   char *theUrlBufP,
		   size_t theUrlBufLen,
		   size_t *theOffsetP,
		   size_t *theLenP) {
  char tempBuf[LEECH_MAX_LINE];
  char cmdBuf[LEECH_MAX_LINE];
  int err, ret, status, urlOffset;

  err = timeout_read_line(&status, 
			  con->sock, 
			  tempBuf,
			  LEECH_MAX_LINE, 
			  LEECH_TIMEOUT);
  if (err || (status <0)) {
    Dprintf("leech_get_cmd::err: %s\n",strerror(errno));
    return LEECH_TCP_ERR;
  }
  if(status==0)
	return LEECH_BYE;	// client disconnected
  ret = sscanf(tempBuf,
		  "%s %d %d %n",
		  cmdBuf,
		  theOffsetP,
		  theLenP,
		  &urlOffset);
  // if were not able to read 3 arguments format of GET command is bad
  if (ret != 3) {
    Dprintf2(3,"leech_get_cmd:: not enough args in get: err=%d,status=%d;ret=%d,buf='%s'\n",err,status,
		    ret,tempBuf);
    return LEECH_ERR;
  }

  // check if command is GET
  if(con->proto == PROTO_LEECH || con->proto == PROTO_LEECH_PERSIST) {
    if (strcmp(cmdBuf, LEECH_GET_STR) != 0) {
      Dprintf2(3,"leech_get_cmd:: command not %s; got '%s' instead\n",
	       LEECH_GET_STR, cmdBuf);
      return LEECH_ERR;
    }
  }
  else if(con->proto == PROTO_HASH_CHECK) {
    if(strcmp(cmdBuf, HASH_GET_STR) != 0) {
      Dprintf2(3,"leech_get_cmd:: command not %s; got '%s' instead\n",
	       HASH_GET_STR, cmdBuf);
    }
  }

  // copy the URL
  strncpy(theUrlBufP, tempBuf+urlOffset, theUrlBufLen);
  if(con->proto == PROTO_LEECH || con->proto == PROTO_LEECH_PERSIST)
    Dprintf2(3, "LEECH CLIENT received: %s\n", theUrlBufP);
  else if(con->proto == PROTO_HASH_CHECK)
    Dprintf2(3, "HASH CHECKER received: %s\n", theUrlBufP);
  return LEECH_OK;
}

