#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "context.h"
#include "updates.h"
#include "source_get.h"
#include "utils.h"


int init(context *);
int cleanUp(context *);

int main(int argc, char** argv) {
	
	// Context Creation	with defaults
	context* c = contextCreate();

	// sodre is tired of SIGPIPES
	signal(SIGPIPE, SIG_IGN);

	// Get the Parameters out and override defaults
	contextParseParameters(c, argc, argv);	// calls usage() on it's own, and exits
	if(!c->URL)
		usage(NULL,NULL);
	init(c);
	Dprintf("DEBUGLEVEL is %d\n",DEBUGLEVEL);
	downloadFile(c);	// where the magic happens
	Dprintf("MAGICAVG %ld\n",time(NULL)-c->startTime);
	Dprintf("Magic Start Time!\n");
	sleep(c->magicTime);
	Dprintf("Magic Time Over\n");
	return cleanUp(c);
}

int cleanUp(context *c){
	float rate;
	time_t  t;
	char * units="B";
	c->stopTime=time(NULL);
	t= c->stopTime- c->startTime;
	rate = (float)c->fileSize/t;
//	contextDestroy(c);
	if(c->quiet)
		return 0;
	if(rate > 1024){
		rate = rate/1024;
		units="KB";
	}
	if(rate > 1024){
		rate = rate/1024;
		units="MB";
	}
	if(rate > 1024){
		rate = rate/1024;
		units="GB";
	}
	printf("\nAverage speed: %.2f %s/S: total time %ld seconds: s/p %d %d %.3f\n", rate, units, t,
			c->sourceBlocksDld, c->peerBlocksDld, (float)c->peerBlocksDld/c->sourceBlocksDld);
			

	return 0;
}


int init(context *c){

	if(c->randomSeed==-1)
		srand48(time(NULL)+(unsigned long )getpid());	// seed the PRNG
	else
		srand48(c->randomSeed);
	c->fileSize = getFileSize(c->URL);
	if(c->fileSize==0){
		fprintf(stderr,"Got filesize of %d; exiting\n",c->fileSize);
		exit(2);
	}
	calcBlocks(c);
	c->dlBitMap = blocksNew(c->blockSize, c->fileSize);
	c->destFileName = contextParseFileName(c);
	c->fd = contextFileOpen(c);

	// Join the server
	if (contextJoin(c) != 0) {
		fprintf(stderr,"Could not join slurpie server %s:%d\n", 
				c->slurpieServerAddress, c->slurpieServerPort);
		exit(1);
	}
	if((c->updateThreadP=spawnSendUpdate(c)) == NULL){
		fprintf(stderr,"Failed to start Update Thread\n");
		exit(2);
	}
	// start leeches thread here
	contextCreateLeechServer(c);
	contextCreateHashCheck(c);
	c->startTime=time(NULL);

	return 0;
}
