#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <pthread.h>

#include "neighbor.h"
#include "updates.h"
#include "connection.h"

pthread_mutex_t neighborMutex= PTHREAD_MUTEX_INITIALIZER;

neighbor* neighborCreate(connection_t* link) {
	neighbor* rv = NULL;
	rv = (neighbor*)malloc(sizeof(neighbor));
	if (rv) {
		rv->link = link;
	}
	else {
		printf("neighborCreate error: Unable to allocate a neighbor structure\n");
	}
	rv->updatesPerRound = DEFAULT_UPDATES_PER_ROUND;
	rv->ref = 1;
	rv->port= connection_get_peer_port(link);
	rv->ip = connection_get_peer_ip(link);
	rv->del_flag = 0;
	return rv;
}
neighbor* neighborRef(neighbor* n) {
	pthread_mutex_lock(&neighborMutex);
	n->ref++;
	pthread_mutex_unlock(&neighborMutex);
	return n;
}

void neighborDestroy(neighbor* d) {
	pthread_mutex_lock(&neighborMutex);
	if (--d->ref == 0) {
		connection_destroy(d->link);
		free(d);
	}
	pthread_mutex_unlock(&neighborMutex);
}

int neighborGetFD(neighbor* n){
	assert(n);
	return connect_get_fd(n->link);
}


int neighborSetUpdates(neighbor* n, int i){
	assert(n);
	n->updatesPerRound=i;
	return i;
}

int neighborGetUpdates(neighbor* n){
	assert(n);
	return n->updatesPerRound;
}

void do_neighbor_tests() {
	// check if ref count is working
	neighbor* t = neighborCreate(NULL);
	neighbor* t_ref = neighborRef(t);
	neighborDestroy(t); t = NULL;
	neighborDestroy(t_ref); t = NULL;
}
