#include <stdio.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "persist_get.h"
#include "leech_protocol.h"
#include "leech_get.h"
#include "adl.h"
#include "utils.h"
#include "source_get.h"


downloadContext * persist_source_download_init(context * c){
	 if(!strncasecmp(c->URL,FTP_PROTO, strlen(FTP_PROTO)))
		 return persist_ftp_get_init(c);
	 if(!strncasecmp(c->URL,WWW_PROTO, strlen(WWW_PROTO)))
		 return persist_www_get_init(c);
	 return NULL;      // protocol not found
}


int  persist_source_download(context * ctx, downloadContext * dlc){
	 if(!strncasecmp(ctx->URL,FTP_PROTO, strlen(FTP_PROTO)))
		 return persist_ftp_get(ctx,dlc);
	 if(!strncasecmp(ctx->URL,WWW_PROTO, strlen(WWW_PROTO)))
		 return persist_www_get(ctx,dlc);
	 return -1;      // protocol not found
}



/* downloadContext * persist_peer_download_init(context * ctx,download * dlc);
 * 	do the openning of the connection, and the initial handshake, but
 * 	leave the actual GET for later
 */
downloadContext * persist_peer_download_init(context * ctx,download * dl){
	connection_t *conn;
	char ipbuf[BUFLEN];
	downloadContext * dlc;
	int err;

	Dprintf("INIT: persist_peer_download_init()\n");
	conn = connection_create(PROTO_LEECH_PERSIST, SLURPIE_VERSION);
	inet_ntop(AF_INET,&dl->ip,ipbuf,BUFLEN);
	err = connection_connect(conn, ipbuf, dl->port);  // use funky connect calls
	if(err){
		Dprintf("persist_peer_download_init:: %s:%d bad connection\n", ipbuf, 
				dl->port);
		connection_destroy(conn);
		updateTreeRevoke(ctx->updateTree,dl->ip,dl->port);
		return NULL;
	}
	dlc = malloc(sizeof(downloadContext));
	assert(dlc);
	dlc->sock  = connect_get_fd(conn);
	dlc->source = 0;	// peer connection, not source
	dlc->lock = malloc(sizeof(pthread_mutex_t));
	dlc->conn = conn;
	dlc->dbuflen=0;
	dlc->offset=0;
	assert(dlc->lock);
	pthread_mutex_init(dlc->lock,NULL);
	dlc->bytes=dlc->current=0;
	dlc->time_start= time(NULL);
	dlc->block = -1;	// not downloading stuff yet
	dlc->obps =0;
	updateTreeMarkLeafBusy(ctx,dl);
	return dlc;
}
int  persist_peer_download(context * ctx,downloadContext * dlc){
	int err,offset, retcode,len;

	Dprintf("CALL: persist_peer_download()\n");
	assert(dlc->source==0);
	offset = dlc->block * ctx->blockSize;
	if(dlc->block == (ctx->numberOfBlocks-1))  // last block
		len = ctx->fileSize - ((ctx->numberOfBlocks-1)*(ctx->blockSize));
	else
		len = ctx->blockSize;
	dlc->dbuflen=0;
	dlc->offset=0;

	err =leech_send_cmd(dlc->conn,ctx->URL,offset,len);
	if(err){
		Dprintf("leech_get:: leech_send_cmd: %d badness \n", err);
		return -1;
	}

	retcode = leech_get_retcode(dlc->conn);
	switch (retcode) {
		case LEECH_CMD_OK:
			break;
		case LEECH_FILE_UNAVAIL:
			Dprintf("persist_peer_download:: File unavailable.\n");
			return LEECH_FILE_UNAVAIL;
			break;
		case LEECH_BYTES_UNAVAIL:
			Dprintf("persist_peer_download:: Bytes unavailable. off=%d, len=%d\n",
					offset, len);
			return LEECH_BYTES_UNAVAIL;
			break;
		case LEECH_SERVICE_UNAVAIL:
			Dprintf("persist_peer_download:: Service unavailable.\n");
			return LEECH_SERVICE_UNAVAIL;
			break;
		default:
			Dprintf("persist_peer_download:: Unknown error from peer %d.\n",retcode);
			return -1;
			break;
	}
	return 0;
}
/********************************************************************/
/* persist_www_get_init:
 * 	make the net connection to the webserver, fill in a download Context,
 * 	and return it
 */
downloadContext * persist_www_get_init(context *c){
	char * hostname, * file;
	int sock;
	unsigned short * port;
	downloadContext *dlc;

	parseURL(c->URL,&hostname,&port,&file);
	if(port==NULL){
		port = malloc(sizeof(unsigned short));	// I hate OTHER PEOPLE's CODE!
		*port=80;
	}
	sock  = make_tcp_connection(hostname,*port);
	if(sock<=0){
		perror("persist_www_get_init::make_tcp_connection");
		return NULL;
	}
	dlc = malloc(sizeof(downloadContext));
	assert(dlc);
	dlc->sock  = sock;
	dlc->source = 1;	// source connection, not peer
	dlc->lock = malloc(sizeof(pthread_mutex_t));
	dlc->conn = NULL;
	dlc->dbuflen=0;
	dlc->offset=0;
	assert(dlc->lock);
	pthread_mutex_init(dlc->lock,NULL);
	dlc->bytes=dlc->current=0;
	dlc->time_start= time(NULL);
	dlc->block = -1;	// not downloading stuff yet
	dlc->obps =0;

	free(hostname);
	free(port);
	free(file);
	return dlc;
}
/* persist_www_get:
 * 	send a persistent HTTP get, and return the dlc 
 */
int persist_www_get(context *c, downloadContext *dlc){
	char * hostname, * file;
	unsigned short * port;
	char ip[BUFLEN];
	char tmpbuf[BUFLEN];
	int err,status,count;
	int offset, len;

	Dprintf("CALL: persist_www_get()\n");
	assert(dlc->source);
	parseURL(c->URL,&hostname,&port,&file);
	hostnameToIpString(hostname,ip,BUFLEN);
	offset = c->blockSize*dlc->block;
	if(dlc->block == (c->numberOfBlocks-1))  // last block
		len = c->fileSize - ((c->numberOfBlocks-1)*(c->blockSize));
	else
		len = c->blockSize;
	dlc->dbuflen=0;
	dlc->offset=0;
	count=snprintf(tmpbuf,BUFLEN,"GET %s HTTP/1.1%s"
			"Host: %s%s"
			"Range: bytes=%d-%d%s%s",
			file, CRLF, ip, CRLF, offset,
			offset+len-1,CRLF,CRLF);
	err=timeout_writeall(&status,dlc->sock,tmpbuf,count,GETTIMEOUT);
	if(err)
		return err;
	if(status<count)
		return status-count;    // short write; must have died
	// read HTTP responds 
	while(!err){
		err = timeout_read_line(&status,dlc->sock,tmpbuf,BUFLEN,GETTIMEOUT);
		if(err)
			return err;
		if(status<0)
			return status;
		if(status==0)
			return -1;
		//if(!strncmp(CONTENT_LENGTH,tmpbuf,strlen(CONTENT_LENGTH)))
		//	sscanf(tmpbuf,CONTENT_LENGTH": %d",&contentLength);
		if(isBlankLine(tmpbuf))
			break;
	}
	return err;
}

/********************************************************************/
downloadContext * persist_ftp_get_init(context *c){
	fprintf(stderr,"Persistent FTP GET's not implemented\n");
	abort();
	return NULL;
}
int persist_ftp_get(context *c, downloadContext *dlc){
	fprintf(stderr,"Persistent FTP GET's not implemented\n");
	abort();
	return 0;
}
