#ifndef BF_H
#define BF_H

#include "types.h"
#include "hashkeys.h"

typedef struct {
  UINT32 sizeBits;  // size of BF in bits
  UINT32 sizeBytes;
  HashKeysCTX hashCTX;
  UINT8  *data;     // bloom filter
} BFCTX;

/* will initialize BFCTX and allocate memory bloom filter data */
/* precodition theSize (size of BF in bits) should be a power of 2 */
void BFInit(BFCTX *theCTX, UINT32 theSize, UINT8 theFuncNum);

/* will deallocate memory taken by bloom filter data */
void BFFree(BFCTX *theCTX);

void BFSet(BFCTX *theCTX, UINT32 thePos);


void BFReset(BFCTX *theCTX, UINT32 thePos);

void BFReseltAll(BFCTX *theCTX);

/* 0 if not set 1 if set */
int BFIsSet(BFCTX *theCTX, UINT32 thePos);

/* will return optimal (minimizes flase positive rate) hash
   function number for a given bloom filter size and set size ratio 
   M - size of bloom filter
   N - size of the set
   theMNRatio = M/N or intuitively number of bits for each element */
UINT8 BFGetOptimalFuncNum(double theMNRatio);

/* will perform unit test of BF code */
void DoBFTest();
#endif
