#ifndef CONNECTION_H
#define CONNECTION_H

#include <unistd.h>
#include "version.h"
#include "protocol.h"
#include "ip_port.h"

extern const char* MAGIC;
typedef struct {
	int sock; // The socket for this connection
	unsigned int timeout; // The default timeout for out socket operations
	int err; // an errno set

	// The protocol that is "talked" on this connection
	protocol_t proto; // The protocol of this side of the connection
	version_t ver; // The version of the protocol on this side
} connection_t ;

// Creates a blank connection_t
connection_t* connection_create(protocol_t proto, version_t ver);

// Calls shutdown on the socket and deletes the cp
void connection_destroy(connection_t* cp);

// Using the protocol specified on the structure, 
// return 0 on success, -1 on error with c->errno set appropriately
int connection_listen(connection_t*, unsigned short port);

// Accepts a new connection on the listening connection c.
// If a protocol was specified on lp, Then a handshake will take place
// in order to detect what the new connection protocol is. 
// If none was specified, all you get back is valid socket.
// Return values:
//   - a valid socket
//   - NULL in case of something weird.
connection_t* connection_accept(connection_t* lp);

// Opens a connection to addr:port 
// Return values:
//   - 0 success
int connection_connect(connection_t* c,  const char* addr, unsigned short port);
int connection_connect_from_port(connection_t* c,  const char* addr, unsigned short port,unsigned short sport);

// Testers
// Returns 1 if sock > 0 && err == 0
int connection_is_valid(connection_t* s);

// Attribute access stuff
// Returns NULL on failure or the pointer to buf
const char* connection_get_peer_addr(connection_t* c, char* buf, int len);
// return the ip in 32bit form, 0 is an error
unsigned int connection_get_peer_ip(connection_t *c);
// Returns the port of the peer 
unsigned short connection_get_peer_port(connection_t* c);
// Returns NULL on failure or the pointer to buf
const char* connection_get_sock_addr(connection_t* c, char* buf, int len);
// Returns the port of the socket 
unsigned short connection_get_sock_port(connection_t* c);

// Methods on Connection
// Gets the next command for connection c 
int connection_get_command_and_parameters(connection_t* c, char* command, size_t len);

// return the socket fd for this connection, so we can select() on it
int connect_get_fd(connection_t* c);

#endif
