#ifndef REM_SLURPIED_H
#define REM_SLURPIED_H

#include <connection.h>


/* *********
 * The Server Protocol API
 * All functions must return 0 on success and -1 otherwise
 * *********/
extern const version_t SLURPIED_VERSION;
typedef enum { UNKNOWN_CMD, ORDER_CMD, GET_DRINKERS_CMD} command_name_t;
const char* command_name__to_string(command_name_t c);
command_name_t command_name_from_string(const char* c);

// An order request. 
// In lame terms, registers with the server the URL for download
typedef struct {
	char* flavor; // What flavor to register/order
	ip_port_t* ip_port;
} order_cmd;
order_cmd* order_create(const char* flavor, const char* addr, unsigned short port);
order_cmd* order_from_string(const char* str);
const char* order_to_string(const order_cmd*, char* buf, int buflen);
void order_destroy(order_cmd**);

// Gets the list of the some people that are downloading the same file
typedef struct {
	char* flavor; // The flavor to get the list of drinkers
	unsigned short num_ip_ports; // The maximum # of drinkers to receive
	ip_port_t** ip_ports; // ip port pairs that is returned
} get_drinkers_cmd;
get_drinkers_cmd* get_drinkers_create(const char* flavor, unsigned short num_ip_ports);
get_drinkers_cmd* get_drinkers_from_string(const char* str);
get_drinkers_cmd* get_drinkers_from_string_no_ip_ports(const char* str);
const char* get_drinkers_to_string(const get_drinkers_cmd*, char* buf, int buflen);
int get_drinkers_set_num_ip_ports(get_drinkers_cmd*, unsigned short num_ip_ports);
void get_drinkers_destroy(get_drinkers_cmd**);

// RPCs from client side
int rem_slurpied_order(connection_t* t, const order_cmd* o);
int rem_slurpied_get_drinkers(connection_t* t, get_drinkers_cmd** d);

// RCPs from server side
int rem_slurpied_get_drinkers_cb(connection_t* t, const get_drinkers_cmd* d);

// Unit tests
void do_rem_slurpied_tests();
#endif
