#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <openssl/sha.h>
#include <openssl/md5.h>

#include <hash.h>

u_int16_t sha_hash(const void* key, size_t keylen) {
	u_int16_t rv = 0;
	char md[SHA_DIGEST_LENGTH];
	char* mdp = md;
	char* fmd = (char*)&rv;
	int i,j;
	SHA1(key, keylen, md);
	// Fold MD into sizeof(unsigned int)
	for(i = 0; i < sizeof(rv); i++, fmd++) {
		for(j = 0; j < SHA_DIGEST_LENGTH/sizeof(rv); j++, mdp++) {
			*fmd ^= *mdp;
		}
	}
	return rv;
}
u_int32_t md5_hash(const void* key, size_t keylen) {
	u_int32_t rv = 0;
	char md[MD5_DIGEST_LENGTH];
	char* mdp = md;
	char* fmd = (char*)&rv;
	int i,j;
	MD5(key, keylen, md);
	// Fold MD into sizeof(unsigned int)
	for(i = 0; i < sizeof(rv); i++, fmd++) {
		for(j = 0; j < MD5_DIGEST_LENGTH/sizeof(rv); j++, mdp++) {
			*fmd ^= *mdp;
		}
	}
	return rv;
}

// Unit test
void do_hash_tests() {
	char* key[] = {
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/MD5SUMS",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-1.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-1_NONUS.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-2.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-3.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-4.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-5.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-6.iso",
	 "ftp://ftp.hr.debian.org/debian-cd/3.0_r0/i386/debian-30r0-i386-binary-7.iso",
	 NULL
	};
	char** keyP;
	printf("Hash Test INIT\n");

	// Test the SHA1 function
	printf("SHA hash\n");
	for (keyP = key; *keyP; keyP++) 
		printf("%s -> %u\n", *keyP, sha_hash(*keyP, strlen(*keyP)));

	// Test the MD5 function
	printf("MD5 hash\n");
	for (keyP = key; *keyP; keyP++) 
		printf("%s -> %u\n", *keyP, md5_hash(*keyP, strlen(*keyP)));


	printf("Hash Test FINISH\n");
}
