#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <openssl/md5.h>
#include <assert.h>

#include "hashkeys.h"

void MDStr(const char *theStr, 
	   UINT8 theDigest[MD5_DIGEST_LENGTH]) 
{
  unsigned int len = strlen(theStr);

  MD5(theStr, len, theDigest);
}

UINT32 GetBits(UINT8 theDigest[MD5_DIGEST_LENGTH],
	       UINT8 theStartIdx,
	       UINT8 theEndIdx) 
{
  UINT8 i;  
  UINT32 result;
  // TBD precondition: theEndIdx - theStartIdx + 1 <= 32
  UINT8 startByte = theStartIdx/8;
  UINT8 startBit = theStartIdx%8;
  UINT8 endByte = theEndIdx/8;
  UINT8 endBit = theEndIdx%8;

  // preconditions
  assert(theStartIdx < theEndIdx);
  assert(theEndIdx - theStartIdx + 1 <= 32);

  if (startByte == endByte) {
    result = (theDigest[startByte] << (7 - endBit)) & 0xff;
    result >>= startBit + (7 - endBit);
  }
  else { 
    // copy the end byte 
    result  = (theDigest[endByte] << (7 - endBit)) & 0xff;
    result >>= (7 - endBit);

    // copy the whole bytes in the middle
    for (i = endByte - 1; i > startByte; --i) {
      result <<= 8;
      result |= theDigest[i];
    }

    // copy the start byte 
    result <<= (7 - startBit + 1);
    result |= theDigest[startByte] >> startBit;
  }

  return result;
}

void HashKeysInit(HashKeysCTX* theCTX,
		  UINT8 theFuncBits,
		  UINT8 theFuncNum) {

  // precondition
  assert(theFuncBits*theFuncNum <= MD5_DIGEST_LENGTH*8);

  theCTX->funcBits = theFuncBits;
  theCTX->funcNum = theFuncNum;
}

void HashKeysCalcInt(HashKeysCTX* theCTX,
		     UINT32 theValue) 
{
  char valueAsStr[20];

  sprintf(valueAsStr, "%ld", theValue);
  HashKeysCalcStr(theCTX, valueAsStr);
}

void HashKeysCalcStr(HashKeysCTX* theCTX,
		     const char* theValue) 
{
  UINT8 digest[MD5_DIGEST_LENGTH];
  UINT8 k;

  // TBD check preconditions

  // calculate MD5 digest for the string
  MDStr(theValue, digest);

  
  // slice the digest
  for (k = 0; k < theCTX->funcNum; ++k) {
    theCTX->keys[k] = GetBits(digest,
			      k*theCTX->funcBits, 
			      (k+1)*theCTX->funcBits-1);
  }
}

void DoHashKeysTest() 
{
  HashKeysCTX ctx;
  UINT32 keyDst0[32];
  UINT32 keyDst1[32];
  UINT32 i;
  UINT8 dummyDigest[MD5_DIGEST_LENGTH] = {0x01, 0x02, 0x03, 0x04,
					  0x11, 0x12, 0x13, 0x14,
					  0x21, 0x22, 0x23, 0x24,
					  0x31, 0x32, 0x33, 0x34};
  // test GetBits
  if (GetBits(dummyDigest, 24, 26) != 0x04) {
    printf("GetBits fails 1\n");
  }

  if (GetBits(dummyDigest, 41, 44) != 0x09) {
    printf("GetBits fails 2\n");
  }

  if (GetBits(dummyDigest, 41, 48) != 0x89) {
    printf("GetBits fails 3\n");
  }

  if (GetBits(dummyDigest, 69, 83) != 0x1911) {
    printf("GetBits fails 4\n");
  }

  if (GetBits(dummyDigest, 96, 127) != 0x34333231) {
    printf("GetBits fails 5\n");
  }

  // test the hashkeys
  HashKeysInit(&ctx, 5, 2);

  for (i = 0; i < 32; ++i) {
    keyDst0[i] = 0;
    keyDst1[i] = 0;
  }

  for (i = 0; i < 32000; ++i) {
    HashKeysCalcInt(&ctx, i);
    ++keyDst0[ctx.keys[0]];
    ++keyDst1[ctx.keys[1]];
  }

  for (i = 0; i < 32; ++i) {
    printf("%ld: %ld %ld\n", i, keyDst0[i], keyDst1[i]);
  }
}
