#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <ip_port.h>

const unsigned int IP_PORT_BUF_LEN = 22;

ip_port_t* ip_port_create(const char* ip, unsigned port) {
	ip_port_t* rv;
	rv = (ip_port_t*)malloc(sizeof(ip_port_t));
	memset(rv,0,sizeof(ip_port_t));
	if (!rv)
		ip_port_destroy(&rv);
	if (rv && ip_port_init(rv, ip, port)) 
		ip_port_destroy(&rv);
	return rv;
}
void ip_port_destroy(ip_port_t** pp) {
	if (*pp) {
		free(*pp);
		*pp = NULL;
	}
}
int ip_port_init(ip_port_t* d, const char* ip, unsigned short port) {
	int rv = -1;
	if (strlen(ip) <= sizeof(d->ip)) {
		strcpy(d->ip, ip);
		d->port = port;
		rv = 0;
	}
	return rv;
}
ip_port_t* ip_port_from_string(const char* str) {
	ip_port_t* rv;
	rv = (ip_port_t*)malloc(sizeof(ip_port_t));
	if (rv) {
		int read;
		unsigned short ip[4];
		memset(rv,0,sizeof(ip_port_t));
		read = sscanf(str, "%hu.%hu.%hu.%hu:%hu", 
				&ip[0], &ip[1], &ip[2], &ip[3], &rv->port);
		if (read == 5 && 
				ip[0] <= 255 && ip[1] <= 255 && ip[2] <= 255 && ip[3] <= 255) {
			snprintf(rv->ip, sizeof(rv->ip), "%hu.%hu.%hu.%hu", ip[0], ip[1], ip[2], ip[3]);
		}
		else
			ip_port_destroy(&rv);
	}
	return rv;
}

const char* ip_port_to_string(const ip_port_t* i, char* buf, size_t len) {
	int written;
	assert(i);
	written = snprintf(buf, len, "%s:%hu", i->ip, i->port);
	if (written < len)
		return buf;
	else
		return NULL;
}

/* ip_port_t* ip_port_duplicate(ip_port_t*);
 * 	duplicate this thinger	-- added by Rob
 */
ip_port_t* ip_port_duplicate(ip_port_t*ip){
	ip_port_t * ret;

	ret = malloc(sizeof(ip_port_t));
	if(!ret){
		perror("ip_port_duplicate:: malloc");
		assert(ret);
	}
	if (ip_port_init(ret, ip->ip, ip->port)){
		free(ret);
		return NULL;
	}
	return ret;
}

/* int ip_port_equal(const ip_port_t*,const ip_port_t*);
 * 	return 1 if these two are the same, 0 otherwise
 */

int ip_port_equal(const ip_port_t*a,const ip_port_t*b){
	int i;
	if(a->port!=b->port)
		return 0;
	i=0;
	while(a->ip[i]!=0){
		if(a->ip[i++]!=b->ip[i])
			return 0;
	}
	return 1;	// must be equal
}



