/*
			QUEUE.C
					Rob Sherwood
					2/9/99 9:16pm
*/


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "queue.h"


/***********/
queuetype *Q_Create(){
	queuetype *p;

	if((p=malloc(sizeof(queuetype))) ==NULL){
		return NULL;
		}
	p->start=NULL;
	p->stop=NULL;

	// initialize lock
	p->lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
	if (p->lock == NULL) {
	  free(p);
	  return NULL;
	}
	pthread_mutex_init(p->lock, NULL); 

	return p;
}

int Q_size(queuetype *p){
	queuedata *d;
	int count =0;
	for(d = p->start; d!= NULL; d = d->next)	
		count++;
	return count;
}

/***********/
int Q_Destroy(queuetype *queue){
	queuedata *p,*q;
	int rv;

	if(queue ==NULL)  return -1;

	q=queue->start;

	while(q !=NULL){
		p=q;
		q=q->next;
		free(p);
	}

	// destroy the lock
        rv = pthread_mutex_destroy(queue->lock);
	// if rv == EBUSY somebody is locking the queue
	// assert(rv != EBUSY) somehow the EBUSY is not defined on my system
	assert(rv == 0);

	free(queue);
	return 0;
}
/***********/
int Q_Empty(queuetype *q){

	if(q==NULL) {
		return -1;
	}
	return ( q->start == NULL)? 1: 0;

}
/*********** adds to end of queue */
int Enqueue(queuetype *q,QUEUE_DATA_THING datum){
	queuedata *neo_datum;	

	if((neo_datum = malloc(sizeof(queuedata))) == NULL){
		return -1;
	}
	neo_datum->data=datum;
	neo_datum->next=NULL;
	if(q->start == NULL){
		q->start=neo_datum;
		q->stop=neo_datum;
	} else {
		q->stop->next=neo_datum;
		q->stop=neo_datum;
	}
	return 0;
}
/********** removes from front of queue */
QUEUE_DATA_THING Dequeue(queuetype *queue){
	queuedata *q;
	QUEUE_DATA_THING datum;

	if( queue ==NULL || queue->start ==NULL)
		return ( QUEUE_DATA_THING )NULL;

	q=queue->start;
	datum=q->data;
	queue->start=q->next;
	free(q); 

	return datum;
}
/**** adds to front of queue **/
int Push(queuetype *q,QUEUE_DATA_THING datum){
	queuedata *neo_datum;	

	if((neo_datum = malloc(sizeof(queuedata))) == NULL){
		return -1;
	}
	neo_datum->data=datum;
	neo_datum->next=q->start;
	q->start=neo_datum;
	return 0;
}

// Elena 07Dec02
QUEUE_DATA_THING Q_Remove(queuetype *theQP, QUEUE_DATA_THING theElmtP) {
  queuedata *d, *prev;
  QUEUE_DATA_THING rv = NULL;

  assert(theQP);

  prev = NULL;
  d = theQP->start;
  while (d != NULL) {
    if (d->data == theElmtP) {
      rv = theElmtP;
      // remove the element from the queue
      if (prev == NULL) {
      // element at head
	theQP->start = d->next;
      }
      else if (d == theQP->stop) {
	// element at tail
	theQP->stop = prev;
	theQP->stop->next = NULL;
      }
      else {
      // element in the middle
	prev->next = d->next;
      }
      // deallocate queue element
      free(d);
      break;
    }
    else {
      prev = d;
      d = d->next;
    }
  }

  return rv;
}

int Q_sizeTS(queuetype *theQP) {
  int rv;

  pthread_mutex_lock(theQP->lock);
  rv = Q_size(theQP);
  pthread_mutex_unlock(theQP->lock);

  return rv;
}

QUEUE_DATA_THING Q_RemoveTS(queuetype *theQP, QUEUE_DATA_THING theElmtP) {
  QUEUE_DATA_THING rv;

  pthread_mutex_lock(theQP->lock);
  rv = Q_Remove(theQP, theElmtP);
  pthread_mutex_unlock(theQP->lock);

  return rv;
}

int Q_EnqueueTS(queuetype *theQP, QUEUE_DATA_THING theElmtP) {
  int rv;

  pthread_mutex_lock(theQP->lock);
  rv = Q_Enqueue(theQP, theElmtP);
  pthread_mutex_unlock(theQP->lock);

  return rv;
}

void do_queue_tests() {
  queuetype *qP = Q_Create();
  int elmts[6] = {0,1,2,3,4,5};
  int i;
  int *elmtP;

  for(i = 0; i < 6; ++i) {
    Q_Enqueue(qP, &elmts[i]);
  }

  // remove from beginning
  elmtP = (int *)Q_Remove(qP, &elmts[0]);
  assert(elmtP!=NULL);
  assert(*elmtP == 0);
  assert(Q_size(qP) == 5);

  // remove from end
  elmtP = (int *)Q_Remove(qP, &elmts[5]);
  assert(elmtP!=NULL);
  assert(*elmtP == 5);
  assert(Q_size(qP) == 4);

  // remove from middle
  elmtP = (int *)Q_Remove(qP, &elmts[3]);
  assert(elmtP!=NULL);
  assert(*elmtP == 3);
  assert(Q_size(qP) == 3);

  // dequeue
  elmtP = (int *)Q_Dequeue(qP);
  assert(elmtP != NULL);
  assert(*elmtP == 1);
  assert(Q_size(qP) == 2);

  // destroy
  Q_Destroy(qP);
}


