#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "rem_slurpied.h"
#include "utils.h"

const char* command_name_to_string(command_name_t c) {
	switch(c) {
		case UNKNOWN_CMD:
			return "UNKNOWN";
		case ORDER_CMD:
			return "ORDER";
		case GET_DRINKERS_CMD:
			return "GET_DRINKERS";
	}
	return "UNKNOWN";
}
command_name_t command_name_from_string(const char* c) {
	command_name_t rv = UNKNOWN_CMD;
	rv = ((strcmp(command_name_to_string(ORDER_CMD), c) == 0) ? ORDER_CMD : rv);
	rv = ((rv == UNKNOWN_CMD && strcmp(command_name_to_string(GET_DRINKERS_CMD), c) == 0) ? GET_DRINKERS_CMD : rv);
	return rv;
}
order_cmd* order_create(const char* flavor, const char* addr, unsigned short port) {
	order_cmd* o = (order_cmd*)malloc(sizeof(order_cmd));
	memset(o, 0, sizeof(order_cmd));
	if (o) {
		o->flavor = strdup(flavor);
		if (!o->flavor) {
			perror("order_create::malloc");
			abort();
		}
		o->ip_port = ip_port_create(addr, port);
		if (!o->ip_port)
			order_destroy(&o);
	}
	return o;
}
order_cmd* order_from_string(const char* str) {
	int read;
	order_cmd* o = (order_cmd*)malloc(sizeof(order_cmd));
	if (o) {
		memset(o, 0, sizeof(order_cmd));
		o->flavor = (char*)malloc(strlen(str));
		if (o->flavor) {
			read = sscanf(str, "%s", o->flavor);
			if (read == 1) {
				o->ip_port = ip_port_from_string(skip_ws(str+strlen(o->flavor)));
				if (!o->ip_port)
					order_destroy(&o);
			}
			else 
				order_destroy(&o);
		}
		else 
			order_destroy(&o);
	}
	return o;
}
const char* order_to_string(const order_cmd* o, char* buf, int buflen) {
	int written;
	char ip_buf[IP_PORT_BUF_LEN];
	written = snprintf(buf, buflen, "%s %s", 
			               o->flavor, ip_port_to_string(o->ip_port, ip_buf, IP_PORT_BUF_LEN));
	if (written < buflen) {
		return buf;
	}
	else 
		return NULL;
}
void order_destroy(order_cmd** o) {
	if (*o) {
		if ((*o)->flavor)
			free((*o)->flavor);
		if ((*o)->ip_port)
			ip_port_destroy(&(*o)->ip_port);
		free(*o);
		*o = NULL;
	}
}
get_drinkers_cmd* get_drinkers_create(const char* flavor, unsigned short num_ip_ports) {
	get_drinkers_cmd* rv = (get_drinkers_cmd*)malloc(sizeof(get_drinkers_cmd));
	if (rv) {
		memset(rv, 0, sizeof(get_drinkers_cmd));
		rv->flavor = strdup(flavor);
		if (rv->flavor) {
			if (get_drinkers_set_num_ip_ports(rv, num_ip_ports) != 0) 
				get_drinkers_destroy(&rv);
		}
		else 
			get_drinkers_destroy(&rv);
	}
	return rv;
}
get_drinkers_cmd* get_drinkers_from_string(const char* str) {
	get_drinkers_cmd* rv = get_drinkers_from_string_no_ip_ports(str);
	if (rv) {
		int i = 0;
		str = skip_nws(str); // skip flavor
		str = skip_ws(str); // white spaces
		str = skip_nws(str); // num_ip_ports
		for (i = 0; i < rv->num_ip_ports && rv ; i++) {
			str = skip_ws(str); // white spaces
			if ((rv->ip_ports[i] = ip_port_from_string(str)) == NULL)
				get_drinkers_destroy(&rv);
			str = skip_nws(str); // the ip and string
		}
	}
	return rv;
}
get_drinkers_cmd* get_drinkers_from_string_no_ip_ports(const char* str) {
	get_drinkers_cmd* rv;
	rv = (get_drinkers_cmd*) malloc(sizeof(get_drinkers_cmd));
	if (rv) {
		memset(rv, 0, sizeof(get_drinkers_cmd));
		rv->flavor = (char*)malloc(strlen(str));
		if (!rv->flavor) 
			get_drinkers_destroy(&rv);
	}
	if (rv && sscanf(str, "%s %hu", rv->flavor, &rv->num_ip_ports) != 2) 
		get_drinkers_destroy(&rv);
	if (rv && get_drinkers_set_num_ip_ports(rv, rv->num_ip_ports) != 0) 
		get_drinkers_destroy(&rv);
	return rv;
}
const char* get_drinkers_to_string(const get_drinkers_cmd* g, char* buf, int buflen) {
	int tot_written = 0;
	const char* rv = NULL;
	int written;
	// Write flavor and num_ip_ports
	written = snprintf(&buf[tot_written], buflen - tot_written, 
			"%s %hu", g->flavor, g->num_ip_ports);
	// Write ip_ports
	if (written < (buflen - tot_written)) {
		int i = 0;
		int still_good = 1;
		tot_written += written;
		for (i = 0; i < g->num_ip_ports && still_good; i++, tot_written+= written) {
			char ip_buf[IP_PORT_BUF_LEN];
			written = snprintf(&buf[tot_written], buflen-tot_written, 
					" %s", ip_port_to_string(g->ip_ports[i], ip_buf, IP_PORT_BUF_LEN));
			still_good = written < (buflen - tot_written);
		}
		rv = (still_good) ? buf : rv;
	}
	return rv;
}
int get_drinkers_set_num_ip_ports(get_drinkers_cmd* d, unsigned short num_ip_ports) {
	int rv = 0;
	// Zap the array first
	if (d->ip_ports) {
		int i;
		for(i = 0; i < d->num_ip_ports; i++)
			ip_port_destroy(&d->ip_ports[i]);
	}
	d->num_ip_ports = num_ip_ports;
	if (d->num_ip_ports != 0) {
		d->ip_ports = (ip_port_t**)malloc(d->num_ip_ports*sizeof(ip_port_t*));
		if (!d->ip_ports) 
			rv = -1;
		else
			memset(d->ip_ports, 0, sizeof(ip_port_t*)*d->num_ip_ports);
	}
	return rv;
}
void get_drinkers_destroy(get_drinkers_cmd** dp) {
	get_drinkers_cmd* d = *dp;
	if (d) {
		if (d->flavor) 
			free(d->flavor);
		// Zap the drinkers array
		get_drinkers_set_num_ip_ports(d, 0);
		free(d);
		*dp = NULL;
	} 
}

// RPCs from client side {{{
int rem_slurpied_order(connection_t* t, const order_cmd* o) {{{
	int rv = 0;
	const char* proto = proto_enum_to_string(t->proto);
	const char* command = command_name_to_string(ORDER_CMD);
	char buf[BUFLEN];
	size_t len;
	//char c;

	assert(t->proto == PROTO_SLURPIED);

	// SEND: MAGIC PROTO ORDER <flavor> <ip:port>
	{
		int status;
		len = snprintf(buf, BUFLEN, "%s %s %s %s %s:%hu\n", MAGIC, proto, command, o->flavor, o->ip_port->ip, o->ip_port->port);
		assert(len < BUFLEN);
		rv = (rv)? rv :timeout_writeall(&status, t->sock, buf, len, t->timeout);
	}
	// RECV: null
#if 0
	{
		int status;
		size_t itm_read;
		char str[2][BUFLEN];
		rv = (rv) ? rv : (timeout_read_line(&status, t->sock, buf, sizeof(buf), t->timeout));
		rv = (rv) ? rv : !((itm_read = sscanf(buf, "%s %s %d%c", str[0], str[1], &c)) == 3 && strcmp(str[0], MAGIC) == 0 && strcmp(str[1], proto) == 0 );
	}
#endif

	if (rv == 0) 
		Dprintf("rem_slurpied_order_flavor %s %s %hu succeded\n", o->flavor, o->ip_port->ip, o->ip_port->port); 
	else
		Dprintf("rem_slurpied_order_flavor %s %s %hu failed\n", o->flavor, o->ip_port->ip, o->ip_port->port); 

	return rv;
}}}
int rem_slurpied_get_drinkers(connection_t* t, get_drinkers_cmd** cp) {
	int err = 0;
	const char* proto = proto_enum_to_string(t->proto);
	const char* command = command_name_to_string(GET_DRINKERS_CMD);
	char buf[BUFLEN];
	size_t len;
	int status;

	// SEND: MAGIC SND_PROTO DRINKERS <flavor> <max drinkers to receive>
	// RECV: <get_drinkers_cmd string>
	get_drinkers_cmd* neo_g=NULL;
	len = snprintf(buf, BUFLEN, "%s %s %s %s %hu\n", MAGIC, proto, command, (*cp)->flavor, (*cp)->num_ip_ports);
	err=timeout_writeall(&status, t->sock, buf, len, t->timeout);
	if(err || status<=0){
		t->err = 1;
		Dprintf("rem_slurpied_get_drinkers failed\n"); 
		return 1;
	}
	err=timeout_read_line(&status, t->sock, buf, BUFLEN, t->timeout);
	if(err || status<=0){
		t->err = 2;
		Dprintf("rem_slurpied_get_drinkers failed\n"); 
		return 2;
	}
	neo_g = get_drinkers_from_string(buf);
	if (neo_g) {
		get_drinkers_destroy(cp);
		*cp = neo_g;
	}
	Dprintf("rem_slurpied_get_drinkers succeded\n"); 
	return 0;
}
// }}}
// RCPs from server side {{{
int rem_slurpied_get_drinkers_cb(connection_t* t, const get_drinkers_cmd* d) {
	int rv = 0;
	char drinkBuf[BUFLEN-1];
	char buf[BUFLEN];
	int status;

	snprintf(buf, BUFLEN, "%s\n",get_drinkers_to_string(d, drinkBuf, BUFLEN-1));

	rv = (rv) ? rv : timeout_writeall(&status, t->sock, buf, strlen(buf), t->timeout);
	if (rv == 0) 
		Dprintf("rem_slurpied_get_drinkers_cb succeed\n");
	else {
		t->err = status;
		Dprintf("rem_slurpied_get_drinkers_cb failed\n");
	}
	return rv;
}
// }}}

// Unit tests
void do_rem_slurpied_tests() {
	char buf[BUFLEN];

	// Order testing
	{
		const char* order_cmd_str = "ftp://bleh 255.0.223.234:3445";
		order_cmd* order;
		Dprintf("order_cmd test START\n");
		order = order_from_string(order_cmd_str);
		assert(order);
		assert(strcmp(order_cmd_str, order_to_string(order, buf, BUFLEN)) == 0);
		order_destroy(&order);
		assert(order == NULL);
		Dprintf("order_cmd test END\n");
	}

	// get_drinkers_testing
	{
		const char* get_drinkers_cmd_str[] = {
			"ftp://bleh 1 0.0.0.1:1",
			"ftp://bleh 2 0.0.0.1:1 0.0.0.2:2",
			NULL
		};
		const char** str = get_drinkers_cmd_str;
		get_drinkers_cmd* get_drinkers;
		Dprintf("get_drinkers_cmd test START\n");
		for (; *str; str++) {
			get_drinkers = get_drinkers_from_string(*str);
			assert(get_drinkers);
			assert(strcmp(*str, get_drinkers_to_string(get_drinkers, buf, BUFLEN)) == 0);
			get_drinkers_destroy(&get_drinkers);
			assert(get_drinkers == NULL);
		}
		Dprintf("get_drinkers_cmd test END\n");
	}
}
