#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <assert.h>
#include <string.h>

#include <utils.h>
#include <command.h>

command_t* command_from_string(const char* cmd_and_params) {
	command_t* c;
	c = (command_t*)malloc(sizeof(command_t));
	if (c) {
		char name [BUFLEN];
		if (sscanf(cmd_and_params, "%s ", name) != 1) 
			command_destroy(&c);
		else {
			const char* params = skip_nws(cmd_and_params); // skip the name
			params = skip_ws(params); // skip the ws
			c->name = command_name_from_string(name);
			switch(c->name) {
				case UNKNOWN_CMD:
					command_destroy(&c);
					break;
				case ORDER_CMD:
					c->u.order = order_from_string(params);
					if (!c->u.order)
						command_destroy(&c);
					break;
				case GET_DRINKERS_CMD:
					c->u.get_drinkers = get_drinkers_from_string_no_ip_ports(params);
					if (!c->u.get_drinkers) 
						command_destroy(&c);
					break;
			};
		}
	}
	return c;
}

void command_destroy(command_t** cp) {
	command_t* c = *cp;
	if (c) {
		switch(c->name) {
			case UNKNOWN_CMD:
				break;
			case ORDER_CMD:
				order_destroy(&c->u.order);
				break;
			case GET_DRINKERS_CMD:
				get_drinkers_destroy(&c->u.get_drinkers);
				break;
		}
		free(c);
		*cp=NULL;
	}
}

// UNIT TEST
void do_command_tests() {
	command_t* t;

	Dprintf("command tests START\n");
	t = command_from_string("ORDER ftp://bleh 255.255.255.255:255");
	assert(t);
	assert(t->name == ORDER_CMD);
	assert(strcmp(t->u.order->flavor, "ftp://bleh") == 0);
	assert(strcmp(t->u.order->ip_port->ip, "255.255.255.255") == 0);
	assert(t->u.order->ip_port->port == 255);
	command_destroy(&t);
	assert(t == NULL);

	t = command_from_string("GET_DRINKERS ftp://bleh 1");
	assert(t);
	assert(t->name == GET_DRINKERS_CMD);
	assert(strcmp(t->u.get_drinkers->flavor, "ftp://bleh") == 0);
	assert(t->u.get_drinkers->num_ip_ports == 1);
	command_destroy(&t);
	assert(t == NULL);

	Dprintf("command tests END\n");
}
