#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <assert.h>
#include <string.h>

#include <utils.h>
#include <protocol.h>
#include <slurpied.h>
#include <command.h>
#include <hash.h>
#include "pmdb.h"

#ifndef MIN
#define MIN(x,y) 	((x)<(y)?(x):(y))
#endif

// Some constants
const static version_t MYVERSION = {0, 1};

// Private helpers
static void slurpied_handle_client(slurpied_t* s, connection_t* c);
static int slurpied_order(slurpied_t*, connection_t*, order_cmd*);
static int slurpied_get_drinkers(slurpied_t*, connection_t*, get_drinkers_cmd*);
/*
static u_int32_t db3_sha_hash(DB* db, const void* key, u_int32_t length) {
	return sha_hash(key, length);
}
*/

slurpied_t* slurpied_create(unsigned short numOfLastKDrinkers) {
	slurpied_t* rv = (slurpied_t*) malloc(sizeof(slurpied_t));
	if (rv == NULL)
		return NULL;
	memset(rv, 0, sizeof(slurpied_t));

	// Set some values
	rv->numOfLastKDrinkers = numOfLastKDrinkers;
	rv->drinkersDB=NULL;
	if ((rv->daemon = connection_create(PROTO_SLURPIED, SLURPIE_VERSION)) == NULL) {
		printf("slurpied_create:connection_create error\n");
		slurpied_destroy(rv);
		rv = 0x0;
	}

	// I'm assuming PMDB here
	if (rv && ((rv->drinkersDB = pmdbNew()) == NULL)) {
		fprintf(stderr,"slurpied_create:pmdbNew()\n");
		slurpied_destroy(rv);
		rv = 0x0;
	}
	return rv;
}
void slurpied_destroy(slurpied_t* d) {
	if (d) {
		if (d->daemon)
			connection_destroy(d->daemon);
		if (d->drinkersDB)
			pmdbDestroy(d->drinkersDB);
		free(d);
	}
}

int slurpied_run(slurpied_t* s) {
	int rv = 0;
	rv = (rv)? rv : (connection_listen(s->daemon, SLURPIED_DEFAULT_PORT));
	if (rv==0) {
		char buf[30];
		connection_t* neo = NULL;
		printf("Slurpie Daemon v%d.%d started at %s:%u\n", 
				    s->daemon->ver.major,	s->daemon->ver.minor, 
						connection_get_sock_addr(s->daemon, buf, 30), connection_get_sock_port(s->daemon));
		// Just keep accepting connections
		while((neo = connection_accept(s->daemon)) != NULL) {
			switch(neo->proto) {
				case PROTO_SLURPIED:
					slurpied_handle_client(s,neo);
					break;
				default:
					connection_destroy(neo);
					break;
			}
		}
	}
	else 
		printf("Slurpie Daemon failed to start\n");
	return -1; 
}
// Private helpers
void slurpied_handle_client(slurpied_t* s, connection_t* c) {
	int err = 0;
	char buf[30];
	char cmd_and_params[BUFLEN];
	printf("New connection from %s\n",connection_get_peer_addr(c, buf, 30));
	while(err == 0 && connection_get_command_and_parameters(c, cmd_and_params, BUFLEN) == 0) {
		command_t* cmd = command_from_string(cmd_and_params);
		if (cmd) {
			switch(cmd->name) {
				case UNKNOWN_CMD:
					Dprintf("slurpied_handle_client: unkown command\n");
					err = 1;
					break;
				case ORDER_CMD:
					err = slurpied_order(s, c, cmd->u.order);
					break;
				case GET_DRINKERS_CMD:
					err = slurpied_get_drinkers(s, c, cmd->u.get_drinkers);
					err = (err) ? err : rem_slurpied_get_drinkers_cb(c, cmd->u.get_drinkers);
					break;
			}
		}
		command_destroy(&cmd);
	}
	printf("Ending connection from %s\n",connection_get_peer_addr(c, buf, 30));
	connection_destroy(c);
}

int slurpied_order(slurpied_t* s, connection_t*c, order_cmd* o) {
	int err = 0;
	char buf[30];
	Dprintf("slurpied_order: %s from %s:%hu\n", o->flavor, o->ip_port->ip, o->ip_port->port);

	if (strcmp(connection_get_peer_addr(c, buf, sizeof(buf)), o->ip_port->ip) == 0 ||
			strcmp(connection_get_peer_addr(c, buf, sizeof(buf)), "127.0.0.1") == 0) {
		drinkersList *dl;
		ip_port_t * ip_port;

		ip_port = malloc(sizeof(ip_port_t));
		if(!ip_port){
			perror("slurpied_order::malloc");
			abort();
		}
		ip_port_init(ip_port, o->ip_port->ip, o->ip_port->port);

		dl = pmdbFind(s->drinkersDB,o->flavor);
		if(!dl){
			// no one has ordered this flavor/URL before
			dl = drinkersListNew(o->flavor,s->numOfLastKDrinkers);
			assert(dl);
			pmdbAdd(s->drinkersDB,o->flavor,dl);
		}
		err =drinkersListAdd(dl,ip_port);	// actually place order
		
	}
	else
		Dprintf("slurpied_order: ignoring host %s behind NAT gateway %s\n", 
				o->ip_port->ip, buf);

	if (err == 0) 
		Dprintf("slurpied_order succeeded\n");
	else
		Dprintf("slurpied_order failed\n");


	return err;
}
/* 	--- old db3 slurpied_get_drinkers():
int slurpied_get_drinkers(slurpied_t* s, connection_t* c, get_drinkers_cmd* d) {
	int err;
	db_recno_t num_duplicates;
	DBT key;
	DBT data;
	DBC* cursor;
	memset(&key, 0, sizeof(DBT));
	key.data = d->flavor;
	key.size = strlen(d->flavor);
	if ((err = s->drinkers->cursor(s->drinkers, NULL, &cursor, 0)) != 0) {
		s->drinkers->err(s->drinkers, err, "slurpied_drinkers: s->drinkers->cursor\n");
	}
	if (!err && (err = cursor->c_get(cursor,&key, &data,DB_SET)) != 0) {
		if (err == DB_NOTFOUND) 
			assert(get_drinkers_set_num_ip_ports(d, 0) == 0);
		else 
			s->drinkers->err(s->drinkers, err, "slurpied_drinkers: cursor->get\n");
	}
	else {
		if ((err = cursor->c_count(cursor, &num_duplicates, 0)) != 0) {
			s->drinkers->err(s->drinkers, err, "slurpied_drinkers: cursor->c_count\n");
		}
		else {
			int i=0;
			const ip_port_t* dp; 
			// Set the d->drinkers ptr and num_drinkers to num_duplicates
			if (num_duplicates < d->num_ip_ports)
				assert(get_drinkers_set_num_ip_ports(d, num_duplicates) == 0);

			// Copy the first data;
			assert(sizeof(ip_port_t) == data.size);
			dp = (ip_port_t*)data.data;
			d->ip_ports[i] = ip_port_create(dp->ip, dp->port);
			i++;

			// Iterate and copy the next flavor duplicates
			for (; i < d->num_ip_ports; i++) {
				// Now go to the next;
				memset(&key, 0, sizeof(DBT));
				memset(&data, 0, sizeof(DBT));
				key.data = d->flavor;
				key.size = strlen(d->flavor);
				if ((err = cursor->c_get(cursor, &key, &data, DB_NEXT_DUP)) != 0) 
					s->drinkers->err(s->drinkers, err, "slurpied_drinkers: cursor->get\n");
				else {
					assert(sizeof(ip_port_t) == data.size);
					dp = (ip_port_t*)data.data;
					d->ip_ports[i] = ip_port_create(dp->ip, dp->port);
				}
			}
		}
	}
	if (cursor && (err = cursor->c_close(cursor)) != 0) {
		s->drinkers->err(s->drinkers, err, "slurpied_drinkers: cursor->destroy\n");
	}
	return err;
}
*/
int slurpied_get_drinkers(slurpied_t* s, connection_t* c, get_drinkers_cmd* d) {
	int err,min,i;
	drinkersList * dl;

	dl = pmdbFind(s->drinkersDB,d->flavor);
	if(!dl){	// no one has ordered this before
		err=get_drinkers_set_num_ip_ports(d, 0);
		assert(err==0);
		return 0;

	}
	min = MIN(dl->length,d->num_ip_ports);	// MIN(what we have, what they want)
	get_drinkers_set_num_ip_ports(d, min);
	for(i=0;i<min;i++)	// copy them over into the get_drinkers_cmd
		d->ip_ports[i]=ip_port_duplicate(dl->data[i]);
	return 0;
}

// Unit Tests
void do_slurpied_tests() {{{

	Dprintf("Slupied Tests: INIT\n");

	// Try to put something on the db

	Dprintf("Slupied Tests: END\n");
}}}
