#!/usr/bin/perl -w

sub usage{
	print STDERR $_ if($_);
	print STDERR "./launch <n> [-f hostlist] [-R] [-r rootdir] [-t  time ] [-s nSkip] -- slurpie_args...\n";
	exit(1);
}

$ROOT="../";
$VERBOSE=0;

@HOSTS=();
$TIME=5;

$SKIP=0;

# Launch n remote processes with the provided args 
# 	of the machines listed in the HOSTS file, seperated by
# 	time seconds; put output in "outdir-n-time-pid/"

$COUNT=shift or usage("need number of hosts\n");
while($ARGV[0] and $ARGV[0] ne "--"){
	if($ARGV[0] eq "-s"){
		shift;
		$SKIP=shift;
		if(!defined($SKIP)){
			 usage("-t needs a parameter\n");
		}
		next;
	}
	if($ARGV[0] eq "-b"){
		$BITTORRENT=1;
		shift @ARGV;
		next;
	}
	if($ARGV[0] eq "-R"){
		$ROGUES=1;
		shift @ARGV;
		next;
	}
	if($ARGV[0] eq "-f"){
		shift @ARGV;
		$HOSTFILE="$ROOT". shift;
		next;
	}
	if($ARGV[0] eq "-v"){
		$VERBOSE=1;
		shift @ARGV;
		next;
	}
	if($ARGV[0] eq "-r"){
		shift;
		$ROOT=shift;
		if(!defined($ROOT)){
			 usage("-r needs a parameter\n");
		}
		next;
	}
	if($ARGV[0] eq "-t"){
		shift;
		$TIME=shift;
		if(!defined($TIME)){
			 usage("-t needs a parameter\n");
		}
		if($TIME<0){
			print "Doing uniform random time [0-59] secs\n";
		}
		next;
	}
	usage("Unknown parameter ".$ARGV[0]);
}

if($VERBOSE){
	print "TIME=$TIME, SKIP=$SKIP, ROOT=$ROOT, VERBOSE=$VERBOSE\n";
}

if($ARGV[0] ne "--"){
	usage("Needs --");
}
shift @ARGV;
if($ROGUES){
	$HOSTFILE="$ROOT/planetlab/hosts.rogues";
	$REMOTE_PARM="-r";
} else {
	if(!$HOSTFILE){
		$HOSTFILE="$ROOT/planetlab/hosts";
	}
	$REMOTE_PARM="";
}

open H, $HOSTFILE or die "open: $HOSTFILE: $!";
for($i=0;$i<$SKIP;$i++){
	<H>;	# skip the first $SKIP hosts
}
for($i=0;$i<$COUNT;$i++){
	$HOSTS[$i]=<H>;	
	if(!$HOSTS[$i]){
		print STDERR "Host file $HOSTFILE does not contain $COUNT hosts\n";
		exit(2);
	}
	chomp $HOSTS[$i];
}

if($SKIP==0){
	$OUTDIR="outdir-$COUNT-$TIME-$$";
} else {
	$OUTDIR="outdir-$COUNT-$TIME-$SKIP-$$";
	
}
mkdir($OUTDIR,0755);
chdir($OUTDIR);
if($BITTORRENT){
	$REMOTE="../$ROOT/remote-bittorrent.pl";
} else {
	$REMOTE="../$ROOT/remote.sh";
}

# launch stuff
$done=0;
for($i=0;$i<$COUNT;$i++){
	$ret = spawn($HOSTS[$i]);
	if($ret> 0){
		$done++;
		if($TIME>=0){
			sleep($TIME);
		} else {
			$t = int(rand(60));	# do uniform random wait
			print "\t	sleeping $t secs\n";
			sleep($t);
		}
	}
}

# wait for stuff
for($i=0;$i<$done;$i++){
	$ret=wait;
	print "LAUNCH ", $done-$i-1, " left :: $PIDS{$ret} ($ret) returned with $?\n";
}
		

sub spawn { 
	$host = shift;
	$OUT="$host.out";
	$uniq=0;
	while(-f $OUT){	 # make unique file names
		$uniq++;
		$OUT="$host.out-$uniq";
	}
	unlink($OUT);
	$pid = fork();
	if(! defined($pid)){
		 warn "fork() failed: $!";
	}
	if($pid == 0){ 	# child
		if($VERBOSE){
			system("sh -x $REMOTE $REMOTE_PARM $host @ARGV 2>&1 | tee $OUT");
		}else {
			system("$REMOTE $REMOTE_PARM $host @ARGV 2>&1 > $OUT");
		}
		exit($?);
	}
	print "Spawned $host to outfile $OUT\n";
	$PIDS{$pid}=$host;
	return $pid;
}
		
