#!/usr/bin/perl -w

sub usage{
	print STDERR $_ if($_);
	print STDERR "./launch <n> [-f hostlist] [-r rootdir] [-t  time ] [-s nSkip] -- command ...\n";
	exit(1);
}

$ROOT="./";
$VERBOSE=0;

@HOSTS=();
$TIME=0;

$SKIP=0;

# Launch n remote processes with the provided args 
# 	of the machines listed in the HOSTS file, seperated by
# 	time seconds; put output in "outdir-n-time-pid/"

$COUNT=shift or usage("need number of hosts\n");
while($ARGV[0] and $ARGV[0] ne "--"){
	if($ARGV[0] eq "-s"){
		shift;
		$SKIP=shift;
		if(!defined($SKIP)){
			 usage("-t needs a parameter\n");
		}
		next;
	}
	if($ARGV[0] eq "-f"){
		shift @ARGV;
		$HOSTFILE="$ROOT". shift;
		next;
	}
	if($ARGV[0] eq "-v"){
		$VERBOSE=1;
		shift @ARGV;
		next;
	}
	if($ARGV[0] eq "-r"){
		shift;
		$ROOT=shift;
		if(!defined($ROOT)){
			 usage("-r needs a parameter\n");
		}
		next;
	}
	if($ARGV[0] eq "-t"){
		shift;
		$TIME=shift;
		if(!defined($TIME)){
			 usage("-t needs a parameter\n");
		}
		next;
	}
	usage("Unknown parameter ".$ARGV[0]);
}

if($VERBOSE){
	print "TIME=$TIME, SKIP=$SKIP, ROOT=$ROOT, VERBOSE=$VERBOSE\n";
}

if($ARGV[0] ne "--"){
	usage("Needs --");
}
shift @ARGV;
if(!$HOSTFILE){
	$HOSTFILE="$ROOT/planetlab/hosts";
}

open H, $HOSTFILE or die "open: $HOSTFILE: $!";
for($i=0;$i<$SKIP;$i++){
	<H>;	# skip the first $SKIP hosts
}
for($i=0;$i<$COUNT;$i++){
	$HOSTS[$i]=<H>;	
	if(!$HOSTS[$i]){
		print STDERR "Host file $HOSTFILE does not contain $COUNT hosts\n";
		exit(2);
	}
	chomp $HOSTS[$i];
}

if($SKIP==0){
	$OUTDIR="outdir-$COUNT-$TIME-$$";
} else {
	$OUTDIR="outdir-$COUNT-$TIME-$SKIP-$$";
	
}
mkdir($OUTDIR,0755);
#chdir($OUTDIR);
# launch stuff
$done=0;
for($i=0;$i<$COUNT;$i++){
	$ret = spawn($HOSTS[$i]);
	if($ret> 0){
		$done++;
		sleep($TIME);
	}
}

# wait for stuff
for($i=0;$i<$done;$i++){
	$ret=wait;
	print "LAUNCH ", $done-$i-1, " left :: $PIDS{$ret} ($ret) returned with $?\n";
}
		

sub spawn { 
	$host = shift;
	my @cmd = copyList(@ARGV);
	update($host,@cmd);
	$OUT="$host.out";
	unlink($OUT);
	$pid = fork();
	if(! defined($pid)){
		 warn "fork() failed: $!";
	}
	if($pid == 0){ 	# child
		if($VERBOSE){
			system("@cmd 2>&1 | tee $OUTDIR/$OUT");
		}else {
			system("@cmd 2>&1 > $OUTDIR/$OUT");
		}
		exit($?);
	}
	print "Spawned $host to outfile $OUT\n";
	$PIDS{$pid}=$host;
	return $pid;
}
		
sub copyList {
	my @dst;
	my $i;
	for($i=0;$i<@_;$i++){
		$dst[$i]=$_[$i];
	}
	return @dst;
}

sub update {
	$host = shift;
	my $i;
	for($i=0;$i<@_;$i++){
		$_[$i]=~ s/%%H/$host/;
	}
	
}
